@@ -15,9 +15,9 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- * 
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
+ *
  * Authors: Martyn Russell <martyn@imendio.com>
  *          Xavier Claessens <xclaesse@gmail.com>
  */
@@ -28,28 +28,48 @@
 
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 
-#include <libmissioncontrol/mission-control.h>
+#include <telepathy-glib/account-manager.h>
 
-#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-utils.h>
 
 #include "empathy-ui-utils.h"
 #include "empathy-account-chooser.h"
 
+#define DEBUG_FLAG EMPATHY_DEBUG_OTHER
+#include <libempathy/empathy-debug.h>
+
+/**
+ * SECTION:empathy-account-chooser
+ * @title:EmpathyAccountChooser
+ * @short_description: A widget used to choose from a list of accounts
+ * @include: libempathy-gtk/empathy-account-chooser.h
+ *
+ * #EmpathyAccountChooser is a widget which extends #GtkComboBox to provide
+ * a chooser of available accounts.
+ */
+
+/**
+ * EmpathyAccountChooser:
+ * @parent: parent object
+ *
+ * Widget which extends #GtkComboBox to provide a chooser of available accounts.
+ */
+
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyAccountChooser)
 typedef struct {
-	EmpathyAccountManager          *manager;
+	TpAccountManager               *manager;
 	gboolean                        set_active_item;
+	gboolean			account_manually_set;
 	gboolean                        has_all_option;
 	EmpathyAccountChooserFilterFunc filter;
 	gpointer                        filter_data;
+	gboolean                        ready;
 } EmpathyAccountChooserPriv;
 
 typedef struct {
 	EmpathyAccountChooser *chooser;
-	McAccount             *account;
+	TpAccount             *account;
 	gboolean               set;
 } SetAccountData;
 
@@ -71,24 +91,26 @@ static void     account_chooser_set_property           (GObject
 							const GValue             *value,
 							GParamSpec               *pspec);
 static void     account_chooser_setup                  (EmpathyAccountChooser    *chooser);
-static void     account_chooser_account_created_cb     (EmpathyAccountManager    *manager,
-							McAccount                *account,
+static void     account_chooser_account_validity_changed_cb (TpAccountManager    *manager,
+							TpAccount                *account,
+							gboolean                  valid,
 							EmpathyAccountChooser    *chooser);
-static void     account_chooser_account_add_foreach    (McAccount                *account,
+static void     account_chooser_account_add_foreach    (TpAccount                *account,
 							EmpathyAccountChooser    *chooser);
-static void     account_chooser_account_deleted_cb     (EmpathyAccountManager    *manager,
-							McAccount                *account,
+static void     account_chooser_account_removed_cb     (TpAccountManager         *manager,
+							TpAccount                *account,
 							EmpathyAccountChooser    *chooser);
-static void     account_chooser_account_remove_foreach (McAccount                *account,
+static void     account_chooser_account_remove_foreach (TpAccount                *account,
 							EmpathyAccountChooser    *chooser);
 static void     account_chooser_update_iter            (EmpathyAccountChooser    *chooser,
 							GtkTreeIter              *iter);
-static void     account_chooser_connection_changed_cb  (EmpathyAccountManager    *manager,
-							McAccount                *account,
-							TpConnectionStatusReason  reason,
-							TpConnectionStatus        new_status,
-							TpConnectionStatus        old_status,
-							EmpathyAccountChooser    *chooser);
+static void     account_chooser_status_changed_cb      (TpAccount  *account,
+							guint       old_status,
+							guint       new_status,
+							guint       reason,
+							gchar      *dbus_error_name,
+							GHashTable *details,
+							gpointer    user_data);
 static gboolean account_chooser_separator_func         (GtkTreeModel             *model,
 							GtkTreeIter              *iter,
 							EmpathyAccountChooser    *chooser);
@@ -102,6 +124,13 @@ enum {
 	PROP_HAS_ALL_OPTION,
 };
 
+enum {
+	READY,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 G_DEFINE_TYPE (EmpathyAccountChooser, empathy_account_chooser, GTK_TYPE_COMBO_BOX);
 
 static void
@@ -113,6 +142,11 @@ empathy_account_chooser_class_init (EmpathyAccountChooserClass *klass)
 	object_class->get_property = account_chooser_get_property;
 	object_class->set_property = account_chooser_set_property;
 
+	/**
+	 * EmpathyAccountChooser:has-all-option:
+	 *
+	 * Have an additional option in the list to mean all accounts.
+	 */
 	g_object_class_install_property (object_class,
 					 PROP_HAS_ALL_OPTION,
 					 g_param_spec_boolean ("has-all-option",
@@ -121,6 +155,16 @@ empathy_account_chooser_class_init (EmpathyAccountChooserClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	signals[READY] =
+		g_signal_new ("ready",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
 	g_type_class_add_private (object_class, sizeof (EmpathyAccountChooserPriv));
 }
 
@@ -132,19 +176,17 @@ empathy_account_chooser_init (EmpathyAccountChooser *chooser)
 
 	chooser->priv = priv;
 	priv->set_active_item = FALSE;
+	priv->account_manually_set = FALSE;
 	priv->filter = NULL;
 	priv->filter_data = NULL;
 
-	priv->manager = empathy_account_manager_dup_singleton ();
+	priv->manager = tp_account_manager_dup ();
 
-	g_signal_connect (priv->manager, "account-created",
-			  G_CALLBACK (account_chooser_account_created_cb),
+	g_signal_connect (priv->manager, "account-validity-changed",
+			  G_CALLBACK (account_chooser_account_validity_changed_cb),
 			  chooser);
-	g_signal_connect (priv->manager, "account-deleted",
-			  G_CALLBACK (account_chooser_account_deleted_cb),
-			  chooser);
-	g_signal_connect (priv->manager, "account-connection-changed",
-			  G_CALLBACK (account_chooser_connection_changed_cb),
+	g_signal_connect (priv->manager, "account-removed",
+			  G_CALLBACK (account_chooser_account_removed_cb),
 			  chooser);
 
 	account_chooser_setup (EMPATHY_ACCOUNT_CHOOSER (chooser));
@@ -156,13 +198,10 @@ account_chooser_finalize (GObject *object)
 	EmpathyAccountChooserPriv *priv = GET_PRIV (object);
 
 	g_signal_handlers_disconnect_by_func (priv->manager,
-					      account_chooser_connection_changed_cb,
+					      account_chooser_account_validity_changed_cb,
 					      object);
 	g_signal_handlers_disconnect_by_func (priv->manager,
-					      account_chooser_account_created_cb,
-					      object);
-	g_signal_handlers_disconnect_by_func (priv->manager,
-					      account_chooser_account_deleted_cb,
+					      account_chooser_account_removed_cb,
 					      object);
 	g_object_unref (priv->manager);
 
@@ -210,6 +249,13 @@ account_chooser_set_property (GObject      *object,
 	};
 }
 
+/**
+ * empathy_account_chooser_new:
+ *
+ * Creates a new #EmpathyAccountChooser.
+ *
+ * Return value: A new #EmpathyAccountChooser
+ */
 GtkWidget *
 empathy_account_chooser_new (void)
 {
@@ -220,11 +266,21 @@ empathy_account_chooser_new (void)
 	return chooser;
 }
 
-McAccount *
-empathy_account_chooser_get_account (EmpathyAccountChooser *chooser)
+/**
+ * empathy_account_chooser_dup_account:
+ * @chooser: an #EmpathyAccountChooser
+ *
+ * Returns the account which is currently selected in the chooser or %NULL
+ * if there is no account selected. The #TpAccount returned should be
+ * unrefed with g_object_unref() when finished with.
+ *
+ * Return value: a new ref to the #TpAccount currently selected, or %NULL.
+ */
+TpAccount *
+empathy_account_chooser_dup_account (EmpathyAccountChooser *chooser)
 {
 	EmpathyAccountChooserPriv *priv;
-	McAccount                *account;
+	TpAccount                 *account;
 	GtkTreeModel             *model;
 	GtkTreeIter               iter;
 
@@ -242,10 +298,57 @@ empathy_account_chooser_get_account (EmpathyAccountChooser *chooser)
 	return account;
 }
 
+/**
+ * empathy_account_chooser_get_connection:
+ * @chooser: an #EmpathyAccountChooser
+ *
+ * Returns a borrowed reference to the #TpConnection associated with the
+ * account currently selected. The caller must reference the returned object with
+ * g_object_ref() if it will be kept
+ *
+ * Return value: a borrowed reference to the #TpConnection associated with the
+ * account curently selected.
+ */
+TpConnection *
+empathy_account_chooser_get_connection (EmpathyAccountChooser *chooser)
+{
+	EmpathyAccountChooserPriv *priv;
+	TpAccount                 *account;
+	TpConnection              *connection;
+
+	g_return_val_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser), NULL);
+
+	priv = GET_PRIV (chooser);
+
+	account = empathy_account_chooser_dup_account (chooser);
+
+	/* if the returned account is NULL, then the account manager probably
+	 * hasn't been prepared yet. It should be safe to return NULL here
+	 * though. */
+	if (account == NULL) {
+		return NULL;
+	}
+
+	connection = tp_account_get_connection (account);
+	g_object_unref (account);
+
+	return connection;
+}
+
+/**
+ * empathy_account_chooser_set_account:
+ * @chooser: an #EmpathyAccountChooser
+ * @account: a #TpAccount
+ *
+ * Sets the currently selected account to @account, if it exists in the list.
+ *
+ * Return value: whether the chooser was set to @account.
+ */
 gboolean
 empathy_account_chooser_set_account (EmpathyAccountChooser *chooser,
-				     McAccount             *account)
+				     TpAccount             *account)
 {
+	EmpathyAccountChooserPriv *priv;
 	GtkComboBox    *combobox;
 	GtkTreeModel   *model;
 	GtkTreeIter     iter;
@@ -253,6 +356,8 @@ empathy_account_chooser_set_account (EmpathyAccountChooser *chooser,
 
 	g_return_val_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser), FALSE);
 
+	priv = GET_PRIV (chooser);
+
 	combobox = GTK_COMBO_BOX (chooser);
 	model = gtk_combo_box_get_model (combobox);
 	gtk_combo_box_get_active_iter (combobox, &iter);
@@ -264,9 +369,21 @@ empathy_account_chooser_set_account (EmpathyAccountChooser *chooser,
 				(GtkTreeModelForeachFunc) account_chooser_set_account_foreach,
 				&data);
 
+	priv->account_manually_set = data.set;
+
 	return data.set;
 }
 
+/**
+ * empathy_account_chooser_get_has_all_option:
+ * @chooser: an #EmpathyAccountChooser
+ *
+ * Returns whether @chooser has the #EmpathyAccountChooser:has-all-option property
+ * set to true.
+ *
+ * Return value: whether @chooser has the #EmpathyAccountChooser:has-all-option property
+ * enabled.
+ */
 gboolean
 empathy_account_chooser_get_has_all_option (EmpathyAccountChooser *chooser)
 {
@@ -275,10 +392,17 @@ empathy_account_chooser_get_has_all_option (EmpathyAccountChooser *chooser)
 	g_return_val_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser), FALSE);
 
 	priv = GET_PRIV (chooser);
-	
+
 	return priv->has_all_option;
 }
 
+/**
+ * empathy_account_chooser_set_has_all_option:
+ * @chooser: an #EmpathyAccountChooser
+ * @has_all_option: a new value for the #EmpathyAccountChooser:has-all-option property
+ *
+ * Sets the #EmpathyAccountChooser:has-all-option property.
+ */
 void
 empathy_account_chooser_set_has_all_option (EmpathyAccountChooser *chooser,
 					   gboolean              has_all_option)
@@ -308,22 +432,22 @@ empathy_account_chooser_set_has_all_option (EmpathyAccountChooser *chooser,
 	 */
 
 	if (has_all_option) {
-		gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (chooser), 
+		gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (chooser),
 						      (GtkTreeViewRowSeparatorFunc)
 						      account_chooser_separator_func,
-						      chooser, 
+						      chooser,
 						      NULL);
 
 		gtk_list_store_prepend (store, &iter);
-		gtk_list_store_set (store, &iter, 
+		gtk_list_store_set (store, &iter,
 				    COL_ACCOUNT_TEXT, NULL,
 				    COL_ACCOUNT_ENABLED, TRUE,
 				    COL_ACCOUNT_POINTER, NULL,
 				    -1);
 
 		gtk_list_store_prepend (store, &iter);
-		gtk_list_store_set (store, &iter, 
-				    COL_ACCOUNT_TEXT, _("All"), 
+		gtk_list_store_set (store, &iter,
+				    COL_ACCOUNT_TEXT, _("All"),
 				    COL_ACCOUNT_ENABLED, TRUE,
 				    COL_ACCOUNT_POINTER, NULL,
 				    -1);
@@ -334,21 +458,55 @@ empathy_account_chooser_set_has_all_option (EmpathyAccountChooser *chooser,
 			}
 		}
 
-		gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (chooser), 
+		gtk_combo_box_set_row_separator_func (GTK_COMBO_BOX (chooser),
 						      (GtkTreeViewRowSeparatorFunc)
 						      NULL,
-						      NULL, 
+						      NULL,
 						      NULL);
 	}
 
 	g_object_notify (G_OBJECT (chooser), "has-all-option");
 }
 
+static void
+account_manager_prepared_cb (GObject *source_object,
+			     GAsyncResult *result,
+			     gpointer user_data)
+{
+	GList *accounts, *l;
+	TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
+	EmpathyAccountChooser *chooser = user_data;
+	EmpathyAccountChooserPriv *priv = GET_PRIV (chooser);
+	GError *error = NULL;
+
+	if (!tp_account_manager_prepare_finish (manager, result, &error)) {
+		DEBUG ("Failed to prepare account manager: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	accounts = tp_account_manager_get_valid_accounts (manager);
+
+	for (l = accounts; l != NULL; l = l->next) {
+		TpAccount *account = l->data;
+
+		account_chooser_account_add_foreach (account, chooser);
+
+		empathy_signal_connect_weak (account, "status-changed",
+					     G_CALLBACK (account_chooser_status_changed_cb),
+					     G_OBJECT (chooser));
+	}
+
+	g_list_free (accounts);
+
+	priv->ready = TRUE;
+	g_signal_emit (chooser, signals[READY], 0);
+}
+
 static void
 account_chooser_setup (EmpathyAccountChooser *chooser)
 {
 	EmpathyAccountChooserPriv *priv;
-	GList                    *accounts;
 	GtkListStore             *store;
 	GtkCellRenderer          *renderer;
 	GtkComboBox              *combobox;
@@ -364,7 +522,7 @@ account_chooser_setup (EmpathyAccountChooser *chooser)
 				    G_TYPE_STRING,    /* Image */
 				    G_TYPE_STRING,    /* Name */
 				    G_TYPE_BOOLEAN,   /* Enabled */
-				    MC_TYPE_ACCOUNT);
+				    TP_TYPE_ACCOUNT);
 
 	gtk_combo_box_set_model (combobox, GTK_TREE_MODEL (store));
 
@@ -384,25 +542,27 @@ account_chooser_setup (EmpathyAccountChooser *chooser)
 					NULL);
 
 	/* Populate accounts */
-	accounts = mc_accounts_list ();
-	g_list_foreach (accounts,
-			(GFunc) account_chooser_account_add_foreach,
-			chooser);
+	tp_account_manager_prepare_async (priv->manager, NULL,
+					  account_manager_prepared_cb, chooser);
 
-	mc_accounts_list_free (accounts);
 	g_object_unref (store);
 }
 
 static void
-account_chooser_account_created_cb (EmpathyAccountManager *manager,
-				    McAccount             *account,
-				    EmpathyAccountChooser *chooser)
+account_chooser_account_validity_changed_cb (TpAccountManager      *manager,
+					     TpAccount             *account,
+					     gboolean               valid,
+					     EmpathyAccountChooser *chooser)
 {
-	account_chooser_account_add_foreach (account, chooser);
+	if (valid) {
+		account_chooser_account_add_foreach (account, chooser);
+	} else {
+		account_chooser_account_remove_foreach (account, chooser);
+	}
 }
 
 static void
-account_chooser_account_add_foreach (McAccount             *account,
+account_chooser_account_add_foreach (TpAccount             *account,
 				     EmpathyAccountChooser *chooser)
 {
 	GtkListStore *store;
@@ -421,15 +581,15 @@ account_chooser_account_add_foreach (McAccount             *account,
 }
 
 static void
-account_chooser_account_deleted_cb (EmpathyAccountManager *manager,
-				    McAccount             *account,
+account_chooser_account_removed_cb (TpAccountManager      *manager,
+				    TpAccount             *account,
 				    EmpathyAccountChooser *chooser)
 {
 	account_chooser_account_remove_foreach (account, chooser);
 }
 
 typedef struct {
-	McAccount   *account;
+	TpAccount   *account;
 	GtkTreeIter *iter;
 	gboolean     found;
 } FindAccountData;
@@ -441,11 +601,11 @@ account_chooser_find_account_foreach (GtkTreeModel *model,
 				      gpointer      user_data)
 {
 	FindAccountData *data = user_data;
-	McAccount       *account;
+	TpAccount  *account;
 
 	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
 
-	if (empathy_account_equal (account, data->account)) {
+	if (account == data->account) {
 		data->found = TRUE;
 		*(data->iter) = *iter;
 		g_object_unref (account);
@@ -460,7 +620,7 @@ account_chooser_find_account_foreach (GtkTreeModel *model,
 
 static gboolean
 account_chooser_find_account (EmpathyAccountChooser *chooser,
-			      McAccount             *account,
+			      TpAccount             *account,
 			      GtkTreeIter           *iter)
 {
 	GtkListStore    *store;
@@ -480,7 +640,7 @@ account_chooser_find_account (EmpathyAccountChooser *chooser,
 }
 
 static void
-account_chooser_account_remove_foreach (McAccount            *account,
+account_chooser_account_remove_foreach (TpAccount             *account,
 					EmpathyAccountChooser *chooser)
 {
 	GtkListStore *store;
@@ -502,7 +662,7 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 	EmpathyAccountChooserPriv *priv;
 	GtkListStore              *store;
 	GtkComboBox               *combobox;
-	McAccount                 *account;
+	TpAccount                 *account;
 	const gchar               *icon_name;
 	gboolean                   is_enabled = TRUE;
 
@@ -515,19 +675,20 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 			    COL_ACCOUNT_POINTER, &account,
 			    -1);
 
-	icon_name = empathy_icon_name_from_account (account);
+	icon_name = tp_account_get_icon_name (account);
 	if (priv->filter) {
 		is_enabled = priv->filter (account, priv->filter_data);
 	}
 
 	gtk_list_store_set (store, iter,
 			    COL_ACCOUNT_IMAGE, icon_name,
-			    COL_ACCOUNT_TEXT, mc_account_get_display_name (account),
+			    COL_ACCOUNT_TEXT, tp_account_get_display_name (account),
 			    COL_ACCOUNT_ENABLED, is_enabled,
 			    -1);
 
 	/* set first connected account as active account */
-	if (priv->set_active_item == FALSE && is_enabled) {
+	if (priv->account_manually_set == FALSE &&
+	    priv->set_active_item == FALSE && is_enabled) {
 		priv->set_active_item = TRUE;
 		gtk_combo_box_set_active_iter (combobox, iter);
 	}
@@ -536,13 +697,15 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 }
 
 static void
-account_chooser_connection_changed_cb (EmpathyAccountManager   *manager,
-				       McAccount               *account,
-				       TpConnectionStatusReason reason,
-				       TpConnectionStatus       new_status,
-				       TpConnectionStatus       old_status,
-				       EmpathyAccountChooser   *chooser)
+account_chooser_status_changed_cb (TpAccount  *account,
+				   guint       old_status,
+				   guint       new_status,
+				   guint       reason,
+				   gchar      *dbus_error_name,
+				   GHashTable *details,
+				   gpointer    user_data)
 {
+	EmpathyAccountChooser *chooser = user_data;
 	GtkTreeIter iter;
 
 	if (account_chooser_find_account (chooser, account, &iter)) {
@@ -560,11 +723,11 @@ account_chooser_separator_func (GtkTreeModel         *model,
 	gboolean                  is_separator;
 
 	priv = GET_PRIV (chooser);
-	
+
 	if (!priv->has_all_option) {
 		return FALSE;
 	}
-	
+
 	gtk_tree_model_get (model, iter, COL_ACCOUNT_TEXT, &text, -1);
 	is_separator = text == NULL;
 	g_free (text);
@@ -578,7 +741,7 @@ account_chooser_set_account_foreach (GtkTreeModel   *model,
 				     GtkTreeIter    *iter,
 				     SetAccountData *data)
 {
-	McAccount *account;
+	TpAccount *account;
 	gboolean   equal;
 
 	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
@@ -587,10 +750,8 @@ account_chooser_set_account_foreach (GtkTreeModel   *model,
 	if ((data->account == NULL) != (account == NULL)) {
 		equal = FALSE;
 	}
-	else if (data->account == account) {
-		equal = TRUE;
-	} else {
-		equal = empathy_account_equal (data->account, account);
+	else {
+		equal = (data->account == account);
 	}
 
 	if (account) {
@@ -619,6 +780,15 @@ account_chooser_filter_foreach (GtkTreeModel *model,
 	return FALSE;
 }
 
+/**
+ * empathy_account_chooser_set_filter:
+ * @chooser: an #EmpathyAccountChooser
+ * @filter: a filter
+ * @user_data: data to pass to @filter, or %NULL
+ *
+ * Sets a filter on the @chooser so only accounts that are %TRUE in the eyes
+ * of the filter are visible in the @chooser.
+ */
 void
 empathy_account_chooser_set_filter (EmpathyAccountChooser           *chooser,
                                     EmpathyAccountChooserFilterFunc  filter,
@@ -640,19 +810,39 @@ empathy_account_chooser_set_filter (EmpathyAccountChooser           *chooser,
 	gtk_tree_model_foreach (model, account_chooser_filter_foreach, chooser);
 }
 
+/**
+ * EmpathyAccountChooserFilterFunc:
+ * @account: a #TpAccount
+ * @user_data: user data, or %NULL
+ *
+ * A function which decides whether the account indicated by @account
+ * is visible.
+ *
+ * Return value: whether the account indicated by @account is visible.
+ */
+
+/**
+ * empathy_account_chooser_filter_is_connected:
+ * @account: a #TpAccount
+ * @user_data: user data or %NULL
+ *
+ * A useful #EmpathyAccountChooserFilterFunc that one could pass into
+ * empathy_account_chooser_set_filter() and only show connected accounts.
+ *
+ * Return value: Whether @account is connected
+ */
 gboolean
-empathy_account_chooser_filter_is_connected (McAccount *account,
+empathy_account_chooser_filter_is_connected (TpAccount *account,
 					     gpointer   user_data)
 {
-	MissionControl     *mc;
-	TpConnectionStatus  status;
-
-	g_return_val_if_fail (MC_IS_ACCOUNT (account), FALSE);
-
-	mc = empathy_mission_control_dup_singleton ();
-	status = mission_control_get_connection_status (mc, account, NULL);
-	g_object_unref (mc);
-
-	return status == TP_CONNECTION_STATUS_CONNECTED;
+	return (tp_account_get_connection_status (account, NULL)
+	    == TP_CONNECTION_STATUS_CONNECTED);
 }
 
+gboolean
+empathy_account_chooser_is_ready (EmpathyAccountChooser *self)
+{
+	EmpathyAccountChooserPriv *priv = GET_PRIV (self);
+
+	return priv->ready;
+}
