@@ -245,6 +245,8 @@ smiley_manager_add_valist (EmpathySmileyManager *manager,
 	}
 
 	/* We give the ownership of path to the smiley */
+	g_object_set_data_full (G_OBJECT (pixbuf), "smiley_str",
+				g_strdup (first_str), g_free);
 	smiley = smiley_new (pixbuf, g_strdup (first_str), path);
 	priv->smileys = g_slist_prepend (priv->smileys, smiley);
 }
@@ -281,20 +283,26 @@ empathy_smiley_manager_load (EmpathySmileyManager *manager)
 
 	/* From fd.o icon-naming spec */
 	empathy_smiley_manager_add (manager, "face-angel",      "O:-)",  "O:)",  NULL);
-        empathy_smiley_manager_add (manager, "face-cool",       "B-)",   "B)",   NULL);
-	empathy_smiley_manager_add (manager, "face-crying",     ":'(", NULL);
+	empathy_smiley_manager_add (manager, "face-angry",      "X-(",   ":@",   NULL);
+	empathy_smiley_manager_add (manager, "face-cool",       "B-)",   "B)",   NULL);
+	empathy_smiley_manager_add (manager, "face-crying",     ":'(",           NULL);
 	empathy_smiley_manager_add (manager, "face-devilish",   ">:-)",  ">:)",  NULL);
-        empathy_smiley_manager_add (manager, "face-embarrassed",":-[",   ":[",   ":-$", ":$", NULL);
+	empathy_smiley_manager_add (manager, "face-embarrassed",":-[",   ":[",   ":-$", ":$", NULL);
 	empathy_smiley_manager_add (manager, "face-kiss",       ":-*",   ":*",   NULL);
+	empathy_smiley_manager_add (manager, "face-laugh",      ":-))",  ":))",  NULL);
 	empathy_smiley_manager_add (manager, "face-monkey",     ":-(|)", ":(|)", NULL);
 	empathy_smiley_manager_add (manager, "face-plain",      ":-|",   ":|",   NULL);
-        empathy_smiley_manager_add (manager, "face-raspberry",  ":-P",   ":P",	 ":-p", ":p", NULL);
+	empathy_smiley_manager_add (manager, "face-raspberry",  ":-P",   ":P",	 ":-p", ":p", NULL);
 	empathy_smiley_manager_add (manager, "face-sad",        ":-(",   ":(",   NULL);
+	empathy_smiley_manager_add (manager, "face-sick",       ":-&",   ":&",   NULL);
 	empathy_smiley_manager_add (manager, "face-smile",      ":-)",   ":)",   NULL);
 	empathy_smiley_manager_add (manager, "face-smile-big",  ":-D",   ":D",   ":-d", ":d", NULL);
 	empathy_smiley_manager_add (manager, "face-smirk",      ":-!",   ":!",   NULL);
-	empathy_smiley_manager_add (manager, "face-surprise",   ":-O",   ":O",   NULL);
+	empathy_smiley_manager_add (manager, "face-surprise",   ":-O",   ":O",   ":-o", ":o", NULL);
+	empathy_smiley_manager_add (manager, "face-tired",      "|-)",   "|)",   NULL);
+	empathy_smiley_manager_add (manager, "face-uncertain",  ":-/",   ":/",   NULL);
 	empathy_smiley_manager_add (manager, "face-wink",       ";-)",   ";)",   NULL);
+	empathy_smiley_manager_add (manager, "face-worried",    ":-S",   ":S",   ":-s", ":s", NULL);
 }
 
 GSList *
