@@ -28,6 +28,9 @@
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/util.h>
 
+#include <gconf/gconf-client.h>
+#include <pango/pango.h>
+#include <gdk/gdk.h>
 
 #include <libempathy/empathy-time.h>
 #include <libempathy/empathy-utils.h>
@@ -43,6 +46,10 @@
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyThemeAdium)
 
+/* GConf key containing current value of font */
+#define EMPATHY_GCONF_FONT_KEY_NAME       "/desktop/gnome/interface/font_name"
+#define BORING_DPI_DEFAULT                96
+
 /* "Join" consecutive messages with timestamps within five minutes */
 #define MESSAGE_JOIN_PERIOD 5*60
 
@@ -54,13 +61,12 @@ typedef struct {
 	gboolean              last_is_backlog;
 	gboolean              page_loaded;
 	GList                *message_queue;
-	gchar                *hovered_uri;
 	guint                 notify_enable_webkit_developer_tools_id;
 	GtkWidget            *inspector_window;
 } EmpathyThemeAdiumPriv;
 
 struct _EmpathyAdiumData {
-	guint  ref_count;
+	gint  ref_count;
 	gchar *path;
 	gchar *basedir;
 	gchar *default_avatar_filename;
@@ -152,141 +158,37 @@ theme_adium_navigation_policy_decision_requested_cb (WebKitWebView             *
 	return TRUE;
 }
 
-static void
-theme_adium_hovering_over_link_cb (EmpathyThemeAdium *theme,
-				   gchar             *title,
-				   gchar             *uri,
-				   gpointer           user_data)
-{
-	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
-
-	if (tp_strdiff (uri, priv->hovered_uri)) {
-		g_free (priv->hovered_uri);
-		priv->hovered_uri = g_strdup (uri);
-	}
-}
-
 static void
 theme_adium_copy_address_cb (GtkMenuItem *menuitem,
 			     gpointer     user_data)
 {
-	EmpathyThemeAdium     *theme = EMPATHY_THEME_ADIUM (user_data);
-	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
+	WebKitHitTestResult   *hit_test_result = WEBKIT_HIT_TEST_RESULT (user_data);
+	gchar                 *uri;
 	GtkClipboard          *clipboard;
 
+	g_object_get (G_OBJECT (hit_test_result), "link-uri", &uri, NULL);
+
 	clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
-	gtk_clipboard_set_text (clipboard, priv->hovered_uri, -1);
+	gtk_clipboard_set_text (clipboard, uri, -1);
 
 	clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
-	gtk_clipboard_set_text (clipboard, priv->hovered_uri, -1);
+	gtk_clipboard_set_text (clipboard, uri, -1);
+
+	g_free (uri);
 }
 
 static void
 theme_adium_open_address_cb (GtkMenuItem *menuitem,
 			     gpointer     user_data)
 {
-	EmpathyThemeAdium     *theme = EMPATHY_THEME_ADIUM (user_data);
-	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
+	WebKitHitTestResult   *hit_test_result = WEBKIT_HIT_TEST_RESULT (user_data);
+	gchar                 *uri;
 
-	empathy_url_show (GTK_WIDGET (menuitem), priv->hovered_uri);
-}
+	g_object_get (G_OBJECT (hit_test_result), "link-uri", &uri, NULL);
 
-static void
-theme_adium_populate_popup_cb (WebKitWebView *view,
-			       GtkMenu       *menu,
-			       gpointer       user_data)
-{
-	EmpathyThemeAdium     *theme = EMPATHY_THEME_ADIUM (view);
-	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
-	GtkWidget             *item;
-	GList                 *items;
-	GtkWidget             *icon;
-	gchar                 *stock_id;
-	gboolean               is_link = FALSE;
-	gboolean               developer_tools_enabled;
+	empathy_url_show (GTK_WIDGET (menuitem), uri);
 
-	/* FIXME: WebKitGTK+'s context menu API clearly needs an
-	 * overhaul.  There is currently no way to know what is being
-	 * clicked, to decide what features to provide. You either
-	 * take what it gives you as a menu, or use hacks to figure
-	 * out what to display. */
-	items = gtk_container_get_children (GTK_CONTAINER (menu));
-	item = GTK_WIDGET (g_list_nth_data (items, 0));
-	g_list_free (items);
-
-	if (GTK_IS_IMAGE_MENU_ITEM (item)) {
-		icon = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (item));
-		gtk_image_get_stock (GTK_IMAGE (icon), &stock_id, NULL);
-
-		if ((!strcmp (stock_id, GTK_STOCK_OPEN)) && priv->hovered_uri)
-			is_link = TRUE;
-	}
-
-	/* Remove default menu items */
-	g_object_get (G_OBJECT (webkit_web_view_get_settings (view)),
-		      "enable-developer-extras", &developer_tools_enabled, NULL);
-	if (!developer_tools_enabled)
-		gtk_container_foreach (GTK_CONTAINER (menu),
-				       (GtkCallback) gtk_widget_destroy, NULL);
-
-	/* Select all item */
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_SELECT_ALL, NULL);
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
-	g_signal_connect_swapped (item, "activate",
-				  G_CALLBACK (webkit_web_view_select_all),
-				  view);
-
-	/* Copy menu item */
-	if (webkit_web_view_can_copy_clipboard (view)) {
-		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_COPY, NULL);
-		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show (item);
-
-		g_signal_connect_swapped (item, "activate",
-					  G_CALLBACK (webkit_web_view_copy_clipboard),
-					  view);
-	}
-
-	/* Clear menu item */
-	item = gtk_separator_menu_item_new ();
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
-	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLEAR, NULL);
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
-	g_signal_connect_swapped (item, "activate",
-				  G_CALLBACK (empathy_chat_view_clear),
-				  view);
-
-	/* We will only add the following menu items if we are
-	 * right-clicking a link */
-	if (!is_link)
-		return;
-
-	/* Separator */
-	item = gtk_separator_menu_item_new ();
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
-	/* Copy Link Address menu item */
-	item = gtk_menu_item_new_with_mnemonic (_("_Copy Link Address"));
-	g_signal_connect (item, "activate",
-			  G_CALLBACK (theme_adium_copy_address_cb),
-			  view);
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
-
-	/* Open Link menu item */
-	item = gtk_menu_item_new_with_mnemonic (_("_Open Link"));
-	g_signal_connect (item, "activate",
-			  G_CALLBACK (theme_adium_open_address_cb),
-			  view);
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
-	gtk_widget_show (item);
+	g_free (uri);
 }
 
 static gchar *
@@ -324,7 +226,10 @@ theme_adium_parse_body (EmpathyThemeAdium *theme,
 			if (s > last) {
 				/* Append the text between last link (or the
 				 * start of the message) and this link */
-				g_string_append_len (string, text + last, s - last);
+				gchar *str;
+				str = g_markup_escape_text (text + last, s - last);
+				g_string_append (string, str);
+				g_free (str);
 			}
 
 			/* Append the link inside <a href=""></a> tag */
@@ -340,9 +245,12 @@ theme_adium_parse_body (EmpathyThemeAdium *theme,
 			last = e;
 		} while (g_match_info_next (match_info, NULL));
 
-		if (e < strlen (text)) {
+		if (e < (gint) strlen (text)) {
 			/* Append the text after the last link */
-			g_string_append_len (string, text + e, strlen (text) - e);
+			gchar *str;
+			str = g_markup_escape_text (text + e, strlen (text) - e);
+			g_string_append (string, str);
+			g_free (str);
 		}
 
 		g_free (ret);
@@ -372,6 +280,8 @@ theme_adium_parse_body (EmpathyThemeAdium *theme,
 
 		g_free (ret);
 		text = ret = g_string_free (string, FALSE);
+	} else {
+		text = ret = g_markup_escape_text (text, -1);
 	}
 
 	g_match_info_free (match_info);
@@ -531,7 +441,7 @@ theme_adium_append_message (EmpathyChatView *view,
 	EmpathyThemeAdium     *theme = EMPATHY_THEME_ADIUM (view);
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
 	EmpathyContact        *sender;
-	EmpathyAccount        *account;
+	TpAccount             *account;
 	gchar                 *dup_body = NULL;
 	const gchar           *body;
 	const gchar           *name;
@@ -557,9 +467,9 @@ theme_adium_append_message (EmpathyChatView *view,
 	sender = empathy_message_get_sender (msg);
 	account = empathy_contact_get_account (sender);
 	service_name = empathy_protocol_name_to_display_name
-		(empathy_account_get_protocol (account));
+		(tp_account_get_protocol (account));
 	if (service_name == NULL)
-		service_name = empathy_account_get_protocol (account);
+		service_name = tp_account_get_protocol (account);
 	timestamp = empathy_message_get_timestamp (msg);
 	body = empathy_message_get_body (msg);
 	dup_body = theme_adium_parse_body (theme, body);
@@ -725,11 +635,15 @@ theme_adium_append_event (EmpathyChatView *view,
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (theme);
 
 	if (priv->data->status_html) {
+		gchar *str_escaped;
+
+		str_escaped = g_markup_escape_text (str, -1);
 		theme_adium_append_html (theme, "appendMessage",
 					 priv->data->status_html,
 					 priv->data->status_len,
-					 str, NULL, NULL, NULL, NULL, "event",
-					 empathy_time_get_current ());
+					 str_escaped, NULL, NULL, NULL, NULL,
+					 "event", empathy_time_get_current ());
+		g_free (str_escaped);
 	}
 
 	/* There is no last contact */
@@ -831,6 +745,111 @@ theme_adium_copy_clipboard (EmpathyChatView *view)
 	webkit_web_view_copy_clipboard (WEBKIT_WEB_VIEW (view));
 }
 
+static void
+theme_adium_context_menu_selection_done_cb (GtkMenuShell *menu, gpointer user_data)
+{
+	WebKitHitTestResult *hit_test_result = WEBKIT_HIT_TEST_RESULT (user_data);
+
+	g_object_unref (hit_test_result);
+}
+
+static void
+theme_adium_context_menu_for_event (EmpathyThemeAdium *theme, GdkEventButton *event)
+{
+	WebKitWebView              *view = WEBKIT_WEB_VIEW (theme);
+	WebKitHitTestResult        *hit_test_result;
+	WebKitHitTestResultContext  context;
+	GtkWidget                  *menu;
+	GtkWidget                  *item;
+
+	hit_test_result = webkit_web_view_get_hit_test_result (view, event);
+	g_object_get (G_OBJECT (hit_test_result), "context", &context, NULL);
+
+	/* The menu */
+	menu = gtk_menu_new ();
+
+	/* Select all item */
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_SELECT_ALL, NULL);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+
+	g_signal_connect_swapped (item, "activate",
+				  G_CALLBACK (webkit_web_view_select_all),
+				  view);
+
+	/* Copy menu item */
+	if (webkit_web_view_can_copy_clipboard (view)) {
+		item = gtk_image_menu_item_new_from_stock (GTK_STOCK_COPY, NULL);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+
+		g_signal_connect_swapped (item, "activate",
+					  G_CALLBACK (webkit_web_view_copy_clipboard),
+					  view);
+	}
+
+	/* Clear menu item */
+	item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+
+	item = gtk_image_menu_item_new_from_stock (GTK_STOCK_CLEAR, NULL);
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+
+	g_signal_connect_swapped (item, "activate",
+				  G_CALLBACK (empathy_chat_view_clear),
+				  view);
+
+	/* We will only add the following menu items if we are
+	 * right-clicking a link */
+	if (context & WEBKIT_HIT_TEST_RESULT_CONTEXT_LINK) {
+		/* Separator */
+		item = gtk_separator_menu_item_new ();
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+
+		/* Copy Link Address menu item */
+		item = gtk_menu_item_new_with_mnemonic (_("_Copy Link Address"));
+		g_signal_connect (item, "activate",
+				  G_CALLBACK (theme_adium_copy_address_cb),
+				  hit_test_result);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+
+		/* Open Link menu item */
+		item = gtk_menu_item_new_with_mnemonic (_("_Open Link"));
+		g_signal_connect (item, "activate",
+				  G_CALLBACK (theme_adium_open_address_cb),
+				  hit_test_result);
+		gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
+	}
+
+	g_signal_connect (GTK_MENU_SHELL (menu), "selection-done",
+			  G_CALLBACK (theme_adium_context_menu_selection_done_cb),
+			  hit_test_result);
+
+	/* Display the menu */
+	gtk_widget_show_all (menu);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+			event->button, event->time);
+}
+
+static gboolean
+theme_adium_button_press_event (GtkWidget *widget, GdkEventButton *event)
+{
+	if (event->button == 3) {
+		gboolean developer_tools_enabled;
+
+		g_object_get (G_OBJECT (webkit_web_view_get_settings (WEBKIT_WEB_VIEW (widget))),
+			      "enable-developer-extras", &developer_tools_enabled, NULL);
+
+		/* We currently have no way to add an inspector menu
+		 * item ourselves, so we disable our customized menu
+		 * if the developer extras are enabled. */
+		if (!developer_tools_enabled) {
+			theme_adium_context_menu_for_event (EMPATHY_THEME_ADIUM (widget), event);
+			return TRUE;
+		}
+	}
+
+	return GTK_WIDGET_CLASS (empathy_theme_adium_parent_class)->button_press_event (widget, event);
+}
+
 static void
 theme_adium_iface_init (EmpathyChatViewIface *iface)
 {
@@ -875,7 +894,6 @@ theme_adium_finalize (GObject *object)
 	EmpathyThemeAdiumPriv *priv = GET_PRIV (object);
 
 	empathy_adium_data_unref (priv->data);
-	g_free (priv->hovered_uri);
 
 	empathy_conf_notify_remove (empathy_conf_get (),
 				    priv->notify_enable_webkit_developer_tools_id);
@@ -971,6 +989,67 @@ theme_adium_inspect_web_view_cb (WebKitWebInspector *inspector,
 	return NULL;
 }
 
+static PangoFontDescription *
+theme_adium_get_default_font (void)
+{
+	GConfClient *gconf_client;
+	PangoFontDescription *pango_fd;
+	gchar *gconf_font_family;
+
+	gconf_client = gconf_client_get_default ();
+	if (gconf_client == NULL) {
+		return NULL;
+	}
+	gconf_font_family = gconf_client_get_string (gconf_client,
+		     EMPATHY_GCONF_FONT_KEY_NAME,
+		     NULL);
+	if (gconf_font_family == NULL) {
+		g_object_unref (gconf_client);
+		return NULL;
+	}
+	pango_fd = pango_font_description_from_string (gconf_font_family);
+	g_free (gconf_font_family);
+	g_object_unref (gconf_client);
+	return pango_fd;
+}
+
+static void
+theme_adium_set_webkit_font (WebKitWebSettings *w_settings,
+			     const gchar *name,
+			     gint size)
+{
+	g_object_set (w_settings, "default-font-family", name, NULL);
+	g_object_set (w_settings, "default-font-size", size, NULL);
+}
+
+static void
+theme_adium_set_default_font (WebKitWebSettings *w_settings)
+{
+	PangoFontDescription *default_font_desc;
+	GdkScreen *current_screen;
+	gdouble dpi = 0;
+	gint pango_font_size = 0;
+
+	default_font_desc = theme_adium_get_default_font ();
+	if (default_font_desc == NULL)
+		return ;
+	pango_font_size = pango_font_description_get_size (default_font_desc)
+		/ PANGO_SCALE ;
+	if (pango_font_description_get_size_is_absolute (default_font_desc)) {
+		current_screen = gdk_screen_get_default ();
+		if (current_screen != NULL) {
+			dpi = gdk_screen_get_resolution (current_screen);
+		} else {
+			dpi = BORING_DPI_DEFAULT;
+		}
+		pango_font_size = (gint) (pango_font_size / (dpi / 72));
+	}
+	theme_adium_set_webkit_font (w_settings,
+		pango_font_description_get_family (default_font_desc),
+		pango_font_size);
+	pango_font_description_free (default_font_desc);
+}
+
 static void
 theme_adium_constructed (GObject *object)
 {
@@ -986,15 +1065,11 @@ theme_adium_constructed (GObject *object)
 	font_family = tp_asv_get_string (priv->data->info, "DefaultFontFamily");
 	font_size = tp_asv_get_int32 (priv->data->info, "DefaultFontSize", NULL);
 	webkit_settings = webkit_web_view_get_settings (webkit_view);
-	if (font_family) {
-		g_object_set (webkit_settings,
-			      "default-font-family", font_family,
-			      NULL);
-	}
-	if (font_size) {
-		g_object_set (webkit_settings,
-			      "default-font-size", font_size,
-			      NULL);
+
+	if (font_family && font_size) {
+		theme_adium_set_webkit_font (webkit_settings, font_family, font_size);
+	} else {
+		theme_adium_set_default_font (webkit_settings);
 	}
 
 	/* Setup webkit inspector */
@@ -1058,6 +1133,7 @@ static void
 empathy_theme_adium_class_init (EmpathyThemeAdiumClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass* widget_class = GTK_WIDGET_CLASS (klass);
 
 	object_class->finalize = theme_adium_finalize;
 	object_class->dispose = theme_adium_dispose;
@@ -1065,6 +1141,8 @@ empathy_theme_adium_class_init (EmpathyThemeAdiumClass *klass)
 	object_class->get_property = theme_adium_get_property;
 	object_class->set_property = theme_adium_set_property;
 
+	widget_class->button_press_event = theme_adium_button_press_event;
+
 	g_object_class_install_property (object_class,
 					 PROP_ADIUM_DATA,
 					 g_param_spec_boxed ("adium-data",
@@ -1094,12 +1172,6 @@ empathy_theme_adium_init (EmpathyThemeAdium *theme)
 	g_signal_connect (theme, "navigation-policy-decision-requested",
 			  G_CALLBACK (theme_adium_navigation_policy_decision_requested_cb),
 			  NULL);
-	g_signal_connect (theme, "populate-popup",
-			  G_CALLBACK (theme_adium_populate_popup_cb),
-			  NULL);
-	g_signal_connect (theme, "hovering-over-link",
-			  G_CALLBACK (theme_adium_hovering_over_link_cb),
-			  NULL);
 
 	priv->notify_enable_webkit_developer_tools_id =
 		empathy_conf_notify_add (empathy_conf_get (),
