@@ -39,7 +39,9 @@ typedef struct {
 	EmpathyContact           *receiver;
 	gchar                    *body;
 	time_t                    timestamp;
+	gboolean                  is_backlog;
 	guint                     id;
+	gboolean                  incoming;
 } EmpathyMessagePriv;
 
 static void empathy_message_finalize   (GObject            *object);
@@ -61,6 +63,8 @@ enum {
 	PROP_RECEIVER,
 	PROP_BODY,
 	PROP_TIMESTAMP,
+	PROP_IS_BACKLOG,
+	PROP_INCOMING,
 };
 
 static void
@@ -112,8 +116,23 @@ empathy_message_class_init (EmpathyMessageClass *class)
 							    G_MAXLONG,
 							    -1,
 							    G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_IS_BACKLOG,
+					 g_param_spec_boolean ("is-backlog",
+							       "History message",
+							       "If the message belongs to history",
+							       FALSE,
+							       G_PARAM_READWRITE));
 
 
+	g_object_class_install_property (object_class,
+					 PROP_INCOMING,
+					 g_param_spec_boolean ("incoming",
+							       "Incoming",
+							       "If this is an incoming (as opposed to sent) message",
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
 	g_type_class_add_private (object_class, sizeof (EmpathyMessagePriv));
 
 }
@@ -170,6 +189,9 @@ message_get_property (GObject    *object,
 	case PROP_BODY:
 		g_value_set_string (value, priv->body);
 		break;
+	case PROP_INCOMING:
+		g_value_set_boolean (value, priv->incoming);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -203,6 +225,9 @@ message_set_property (GObject      *object,
 		empathy_message_set_body (EMPATHY_MESSAGE (object),
 					 g_value_get_string (value));
 		break;
+	case PROP_INCOMING:
+		priv->incoming = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -328,28 +353,15 @@ empathy_message_set_body (EmpathyMessage *message,
 			  const gchar    *body)
 {
 	EmpathyMessagePriv       *priv = GET_PRIV (message);
-	TpChannelTextMessageType  type;
 
 	g_return_if_fail (EMPATHY_IS_MESSAGE (message));
 
 	g_free (priv->body);
-	priv->body = NULL;
-
-	type = TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL;
-	if (g_str_has_prefix (body, "/me")) {
-		type = TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION;
-		body += 4;
-	}
-	else if (g_str_has_prefix (body, "/say")) {
-		body += 5;
-	}
 
 	if (body) {
 		priv->body = g_strdup (body);
-	}
-
-	if (type != priv->type) {
-		empathy_message_set_tptype (message, type);
+	} else {
+		priv->body = NULL;
 	}
 
 	g_object_notify (G_OBJECT (message), "body");
@@ -387,6 +399,33 @@ empathy_message_set_timestamp (EmpathyMessage *message,
 	g_object_notify (G_OBJECT (message), "timestamp");
 }
 
+gboolean
+empathy_message_is_backlog (EmpathyMessage *message)
+{
+	EmpathyMessagePriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_MESSAGE (message), FALSE);
+
+	priv = GET_PRIV (message);
+
+	return priv->is_backlog;
+}
+
+void
+empathy_message_set_is_backlog (EmpathyMessage *message,
+				gboolean is_backlog)
+{
+	EmpathyMessagePriv *priv;
+
+	g_return_if_fail (EMPATHY_IS_MESSAGE (message));
+
+	priv = GET_PRIV (message);
+
+	priv->is_backlog = is_backlog;
+
+	g_object_notify (G_OBJECT (message), "is-backlog");
+}
+
 #define IS_SEPARATOR(ch) (ch == ' ' || ch == ',' || ch == '.' || ch == ':')
 gboolean
 empathy_message_should_highlight (EmpathyMessage *message)
@@ -500,6 +539,30 @@ empathy_message_set_id (EmpathyMessage *message, guint id)
 	priv->id = id;
 }
 
+void
+empathy_message_set_incoming (EmpathyMessage *message, gboolean incoming)
+{
+	EmpathyMessagePriv *priv;
+
+	g_return_if_fail (EMPATHY_IS_MESSAGE (message));
+
+	priv = GET_PRIV (message);
+
+	priv->incoming = incoming;
+
+	g_object_notify (G_OBJECT (message), "incoming");
+}
+
+gboolean
+empathy_message_is_incoming (EmpathyMessage *message)
+{
+	EmpathyMessagePriv *priv = GET_PRIV (message);
+
+	g_return_val_if_fail (EMPATHY_IS_MESSAGE (message), FALSE);
+
+	return priv->incoming;
+}
+
 gboolean
 empathy_message_equal (EmpathyMessage *message1, EmpathyMessage *message2)
 {
