@@ -1069,18 +1069,35 @@ get_contact_by_handle_cb (TpConnection *connection,
 {
 	GetContactsData *data = user_data;
 	EmpathyContact  *contact = NULL;
+	GError *err = NULL;
 
 	if (n_contacts == 1) {
 		contact = dup_contact_for_tp_contact (data->tp_factory,
 						      contacts[0]);
 	}
+	else {
+		if (error == NULL) {
+			/* tp-glib will provide an error only if the whole operation failed,
+			 * but not if, for example, the handle was invalid. We create an error
+			 * so the caller of empathy_tp_contact_factory_get_from_handle can
+			 * rely on the error to check if the operation succeeded or not. */
+
+			err = g_error_new_literal (TP_ERRORS, TP_ERROR_INVALID_HANDLE,
+						      "handle is invalid");
+		}
+		else {
+			err = g_error_copy (error);
+		}
+	}
 
 	if (data->callback.contact_cb) {
 		data->callback.contact_cb (data->tp_factory,
 				           contact,
-				           error,
+				           err,
 				           data->user_data, weak_object);
 	}
+
+	g_clear_error (&err);
 }
 
 void
@@ -1281,6 +1298,58 @@ tp_contact_factory_finalize (GObject *object)
 	G_OBJECT_CLASS (empathy_tp_contact_factory_parent_class)->finalize (object);
 }
 
+static void
+connection_ready_cb (TpConnection *connection,
+				const GError *error,
+				gpointer user_data)
+{
+	EmpathyTpContactFactory *tp_factory = EMPATHY_TP_CONTACT_FACTORY (user_data);
+	EmpathyTpContactFactoryPriv *priv = GET_PRIV (tp_factory);
+
+	if (error != NULL)
+		goto out;
+
+	/* FIXME: This should be moved to TpContact */
+	tp_cli_connection_interface_avatars_connect_to_avatar_updated (priv->connection,
+								       tp_contact_factory_avatar_updated_cb,
+								       NULL, NULL,
+								       G_OBJECT (tp_factory),
+								       NULL);
+	tp_cli_connection_interface_avatars_connect_to_avatar_retrieved (priv->connection,
+									 tp_contact_factory_avatar_retrieved_cb,
+									 NULL, NULL,
+									 G_OBJECT (tp_factory),
+									 NULL);
+	tp_cli_connection_interface_capabilities_connect_to_capabilities_changed (priv->connection,
+										  tp_contact_factory_capabilities_changed_cb,
+										  NULL, NULL,
+										  G_OBJECT (tp_factory),
+										  NULL);
+
+
+	tp_cli_connection_interface_location_connect_to_location_updated (priv->connection,
+									   tp_contact_factory_location_updated_cb,
+									   NULL, NULL,
+									   G_OBJECT (tp_factory),
+									   NULL);
+
+
+	tp_cli_connection_interface_avatars_call_get_avatar_requirements (priv->connection,
+									  -1,
+									  tp_contact_factory_got_avatar_requirements_cb,
+									  NULL, NULL,
+									  G_OBJECT (tp_factory));
+
+	tp_cli_dbus_properties_call_get (priv->connection, -1,
+		TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
+		"RequestableChannelClasses",
+		get_requestable_channel_classes_cb, NULL, NULL,
+		G_OBJECT (tp_factory));
+
+out:
+	g_object_unref (tp_factory);
+}
+
 static GObject *
 tp_contact_factory_constructor (GType                  type,
 				guint                  n_props,
@@ -1292,41 +1361,11 @@ tp_contact_factory_constructor (GType                  type,
 	tp_factory = G_OBJECT_CLASS (empathy_tp_contact_factory_parent_class)->constructor (type, n_props, props);
 	priv = GET_PRIV (tp_factory);
 
-	/* FIXME: This should be moved to TpContact */
-	tp_cli_connection_interface_avatars_connect_to_avatar_updated (priv->connection,
-								       tp_contact_factory_avatar_updated_cb,
-								       NULL, NULL,
-								       tp_factory,
-								       NULL);
-	tp_cli_connection_interface_avatars_connect_to_avatar_retrieved (priv->connection,
-									 tp_contact_factory_avatar_retrieved_cb,
-									 NULL, NULL,
-									 tp_factory,
-									 NULL);
-	tp_cli_connection_interface_capabilities_connect_to_capabilities_changed (priv->connection,
-										  tp_contact_factory_capabilities_changed_cb,
-										  NULL, NULL,
-										  tp_factory,
-										  NULL);
-
-
-	tp_cli_connection_interface_location_connect_to_location_updated (priv->connection,
-									   tp_contact_factory_location_updated_cb,
-									   NULL, NULL,
-									   G_OBJECT (tp_factory),
-									   NULL);
-
-	/* FIXME: This should be moved to TpConnection */
-	tp_cli_connection_interface_avatars_call_get_avatar_requirements (priv->connection,
-									  -1,
-									  tp_contact_factory_got_avatar_requirements_cb,
-									  NULL, NULL,
-									  tp_factory);
-	tp_cli_dbus_properties_call_get (priv->connection, -1,
-		TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
-		"RequestableChannelClasses",
-		get_requestable_channel_classes_cb, NULL, NULL,
-		G_OBJECT (tp_factory));
+	/* Ensure to keep the self object alive while the call_when_ready is
+	 * running */
+	g_object_ref (tp_factory);
+	tp_connection_call_when_ready (priv->connection, connection_ready_cb,
+		tp_factory);
 
 	return tp_factory;
 }
