@@ -105,6 +105,7 @@ struct _EmpathyCallWindowPriv
   gboolean outgoing;
 
   GtkUIManager *ui_manager;
+  GtkWidget *errors_vbox;
   GtkWidget *video_output;
   GtkWidget *video_preview;
   GtkWidget *remote_user_avatar_widget;
@@ -697,6 +698,7 @@ empathy_call_window_init (EmpathyCallWindow *self)
   filename = empathy_file_lookup ("empathy-call-window.ui", "src");
   gui = empathy_builder_get_file (filename,
     "call_window_vbox", &top_vbox,
+    "errors_vbox", &priv->errors_vbox,
     "pane", &priv->pane,
     "statusbar", &priv->statusbar,
     "redial", &priv->redial_button,
@@ -1447,19 +1449,204 @@ empathy_call_window_update_timer (gpointer user_data)
   EmpathyCallWindow *self = EMPATHY_CALL_WINDOW (user_data);
   EmpathyCallWindowPriv *priv = GET_PRIV (self);
   gchar *str;
-  gdouble time;
+  gdouble time_;
 
-  time = g_timer_elapsed (priv->timer, NULL);
+  time_ = g_timer_elapsed (priv->timer, NULL);
 
   /* Translators: number of minutes:seconds the caller has been connected */
-  str = g_strdup_printf (_("Connected — %d:%02dm"), (int) time / 60,
-    (int) time % 60);
+  str = g_strdup_printf (_("Connected — %d:%02dm"), (int) time_ / 60,
+    (int) time_ % 60);
   empathy_call_window_status_message (self, str);
   g_free (str);
 
   return TRUE;
 }
 
+static void
+display_error (EmpathyCallWindow *self,
+    EmpathyTpCall *call,
+    const gchar *img,
+    const gchar *title,
+    const gchar *desc,
+    const gchar *details)
+{
+  EmpathyCallWindowPriv *priv = GET_PRIV (self);
+  GtkWidget *info_bar;
+  GtkWidget *content_area;
+  GtkWidget *hbox;
+  GtkWidget *vbox;
+  GtkWidget *image;
+  GtkWidget *label;
+  gchar *txt;
+
+  /* Create info bar */
+  info_bar = gtk_info_bar_new_with_buttons (GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+      NULL);
+
+  gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_WARNING);
+
+  content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar));
+
+  /* hbox containing the image and the messages vbox */
+  hbox = gtk_hbox_new (FALSE, 3);
+  gtk_container_add (GTK_CONTAINER (content_area), hbox);
+
+  /* Add image */
+  image = gtk_image_new_from_icon_name (img, GTK_ICON_SIZE_DIALOG);
+  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+
+  /* vbox containing the main message and the details expander */
+  vbox = gtk_vbox_new (FALSE, 3);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+
+  /* Add text */
+  txt = g_strdup_printf ("<b>%s</b>\n%s", title, desc);
+
+  label = gtk_label_new (NULL);
+  gtk_label_set_markup (GTK_LABEL (label), txt);
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+  g_free (txt);
+
+  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+
+  /* Add details */
+  if (details != NULL)
+    {
+      GtkWidget *expander;
+
+      expander = gtk_expander_new (_("Technical Details"));
+
+      txt = g_strdup_printf ("<i>%s</i>", details);
+
+      label = gtk_label_new (NULL);
+      gtk_label_set_markup (GTK_LABEL (label), txt);
+      gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+      gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+      g_free (txt);
+
+      gtk_container_add (GTK_CONTAINER (expander), label);
+      gtk_box_pack_start (GTK_BOX (vbox), expander, TRUE, TRUE, 0);
+    }
+
+  g_signal_connect (info_bar, "response",
+      G_CALLBACK (gtk_widget_destroy), NULL);
+
+  gtk_box_pack_start (GTK_BOX (priv->errors_vbox), info_bar,
+      FALSE, FALSE, CONTENT_HBOX_CHILDREN_PACKING_PADDING);
+  gtk_widget_show_all (info_bar);
+}
+
+static gchar *
+media_stream_error_to_txt (EmpathyCallWindow *self,
+    EmpathyTpCall *call,
+    gboolean audio,
+    TpMediaStreamError error)
+{
+  EmpathyCallWindowPriv *priv = GET_PRIV (self);
+  const gchar *cm;
+  gchar *url;
+  gchar *result;
+
+  switch (error)
+    {
+      case TP_MEDIA_STREAM_ERROR_CODEC_NEGOTIATION_FAILED:
+        if (audio)
+          return g_strdup_printf (
+              _("%s's software does not understand any of the audio formats "
+                "supported by your computer"),
+            empathy_contact_get_name (priv->contact));
+        else
+          return g_strdup_printf (
+              _("%s's software does not understand any of the video formats "
+                "supported by your computer"),
+            empathy_contact_get_name (priv->contact));
+
+      case TP_MEDIA_STREAM_ERROR_CONNECTION_FAILED:
+        return g_strdup_printf (
+            _("Can't establish a connection to %s. "
+              "One of you might be on a network that does not allow "
+              "direct connections."),
+          empathy_contact_get_name (priv->contact));
+
+      case TP_MEDIA_STREAM_ERROR_NETWORK_ERROR:
+          return g_strdup (_("There was a failure on the network"));
+
+      case TP_MEDIA_STREAM_ERROR_NO_CODECS:
+        if (audio)
+          return g_strdup (_("The audio formats necessary for this call "
+                "are not installed on your computer"));
+        else
+          return g_strdup (_("The video formats necessary for this call "
+                "are not installed on your computer"));
+
+      case TP_MEDIA_STREAM_ERROR_INVALID_CM_BEHAVIOR:
+        cm = empathy_tp_call_get_connection_manager (call);
+
+        url = g_strdup_printf ("http://bugs.freedesktop.org/enter_bug.cgi?"
+            "product=Telepathy&amp;component=%s", cm);
+
+        result = g_strdup_printf (
+            _("Something not expected happened in a Telepathy component. "
+              "Please <a href=\"%s\">report this bug</a> and attach "
+              "logs gathered from the 'Debug' window in the Help menu."), url);
+
+        g_free (url);
+        return result;
+
+      case TP_MEDIA_STREAM_ERROR_MEDIA_ERROR:
+        return g_strdup (_("There was a failure in the call engine"));
+
+      default:
+        return NULL;
+    }
+}
+
+static void
+empathy_call_window_stream_error (EmpathyCallWindow *self,
+    EmpathyTpCall *call,
+    gboolean audio,
+    guint code,
+    const gchar *msg,
+    const gchar *icon,
+    const gchar *title)
+{
+  gchar *desc;
+
+  desc = media_stream_error_to_txt (self, call, audio, code);
+  if (desc == NULL)
+    {
+      /* No description, use the error message. That's not great as it's not
+       * localized but it's better than nothing. */
+      display_error (self, call, icon, title, msg, NULL);
+    }
+  else
+    {
+      display_error (self, call, icon, title, desc, msg);
+      g_free (desc);
+    }
+}
+
+static void
+empathy_call_window_audio_stream_error (EmpathyTpCall *call,
+    guint code,
+    const gchar *msg,
+    EmpathyCallWindow *self)
+{
+  empathy_call_window_stream_error (self, call, TRUE, code, msg,
+      "gnome-stock-mic", _("Can't establish audio stream"));
+}
+
+static void
+empathy_call_window_video_stream_error (EmpathyTpCall *call,
+    guint code,
+    const gchar *msg,
+    EmpathyCallWindow *self)
+{
+  empathy_call_window_stream_error (self, call, FALSE, code, msg,
+      "camera-web", _("Can't establish video stream"));
+}
+
 static gboolean
 empathy_call_window_connected (gpointer user_data)
 {
@@ -1752,10 +1939,31 @@ empathy_call_window_update_avatars_visibility (EmpathyTpCall *call,
   empathy_call_window_update_self_avatar_visibility (window);
 }
 
+static void
+call_handler_notify_tp_call_cb (EmpathyCallHandler *handler,
+    GParamSpec *spec,
+    EmpathyCallWindow *self)
+{
+  EmpathyCallWindowPriv *priv = GET_PRIV (self);
+  EmpathyTpCall *call;
+
+  g_object_get (priv->handler, "tp-call", &call, NULL);
+  if (call == NULL)
+    return;
+
+  empathy_signal_connect_weak (call, "audio-stream-error",
+      G_CALLBACK (empathy_call_window_audio_stream_error), G_OBJECT (self));
+  empathy_signal_connect_weak (call, "video-stream-error",
+      G_CALLBACK (empathy_call_window_video_stream_error), G_OBJECT (self));
+
+  g_object_unref (call);
+}
+
 static void
 empathy_call_window_realized_cb (GtkWidget *widget, EmpathyCallWindow *window)
 {
   EmpathyCallWindowPriv *priv = GET_PRIV (window);
+  EmpathyTpCall *call;
 
   g_signal_connect (priv->handler, "conference-added",
     G_CALLBACK (empathy_call_window_conference_added_cb), window);
@@ -1770,6 +1978,24 @@ empathy_call_window_realized_cb (GtkWidget *widget, EmpathyCallWindow *window)
   g_signal_connect (priv->handler, "stream-closed",
     G_CALLBACK (empathy_call_window_channel_stream_closed_cb), window);
 
+  g_object_get (priv->handler, "tp-call", &call, NULL);
+  if (call != NULL)
+    {
+      empathy_signal_connect_weak (call, "audio-stream-error",
+        G_CALLBACK (empathy_call_window_audio_stream_error), G_OBJECT (window));
+      empathy_signal_connect_weak (call, "video-stream-error",
+        G_CALLBACK (empathy_call_window_video_stream_error), G_OBJECT (window));
+
+      g_object_unref (call);
+    }
+  else
+    {
+      /* tp-call doesn't exist yet, we'll connect signals once it has been
+       * set */
+      g_signal_connect (priv->handler, "notify::tp-call",
+        G_CALLBACK (call_handler_notify_tp_call_cb), window);
+    }
+
   gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
 }
 
