@@ -23,6 +23,7 @@
  *          Martyn Russell <martyn@imendio.com>
  *          Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  *          Xavier Claessens <xclaesse@gmail.com>
+ *          Rômulo Fernandes Machado <romulo@castorgroup.net>
  */
 
 #include <config.h>
@@ -34,13 +35,13 @@
 #include <glib/gi18n.h>
 #include <libnotify/notification.h>
 
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-message.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-chatroom-manager.h>
-#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-utils.h>
 
 #include <libempathy-gtk/empathy-images.h>
@@ -51,10 +52,10 @@
 #include <libempathy-gtk/empathy-smiley-manager.h>
 #include <libempathy-gtk/empathy-sound.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
+#include <libempathy-gtk/empathy-notify-manager.h>
 
 #include "empathy-chat-window.h"
 #include "empathy-about-dialog.h"
-#include "empathy-misc.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_CHAT
 #include <libempathy/empathy-debug.h>
@@ -72,8 +73,8 @@ typedef struct {
 	GList       *chats_composing;
 	gboolean     page_added;
 	gboolean     dnd_same_window;
-	guint        save_geometry_id;
 	EmpathyChatroomManager *chatroom_manager;
+	EmpathyNotifyManager *notify_mgr;
 	GtkWidget   *dialog;
 	GtkWidget   *notebook;
 	NotifyNotification *notification;
@@ -308,47 +309,41 @@ _submenu_notify_visible_changed_cb (GObject    *object,
 }
 
 static void
-chat_window_update (EmpathyChatWindow *window)
+chat_window_menu_context_update (EmpathyChatWindowPriv *priv,
+			      gint num_pages)
 {
-	EmpathyChatWindowPriv *priv = GET_PRIV (window);
-	gboolean               first_page;
-	gboolean               last_page;
-	gboolean               is_connected;
-	gint                   num_pages;
-	gint                   page_num;
-	gint                   i;
-	const gchar           *name;
-	guint                  n_chats;
-	GdkPixbuf             *icon;
-	EmpathyContact        *remote_contact;
-	gboolean               avatar_in_icon;
-	GtkWidget             *chat;
-	GtkWidget             *chat_close_button;
-	GtkWidget             *menu, *submenu, *orig_submenu;
+	gboolean first_page;
+	gboolean last_page;
+	gboolean is_connected;
+	gint     page_num;
 
-	/* Get information */
 	page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
-	num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
 	first_page = (page_num == 0);
 	last_page = (page_num == (num_pages - 1));
 	is_connected = empathy_chat_get_tp_chat (priv->current_chat) != NULL;
-	name = empathy_chat_get_name (priv->current_chat);
-	n_chats = g_list_length (priv->chats);
 
-	DEBUG ("Update window");
+	DEBUG ("Update window : Menu Contexts (Tabs & Conv)");
 
-	/* Update menu */
-	gtk_action_set_sensitive (priv->menu_tabs_next, !last_page);
-	gtk_action_set_sensitive (priv->menu_tabs_prev, !first_page);
+	gtk_action_set_sensitive (priv->menu_tabs_next, TRUE);
+	gtk_action_set_sensitive (priv->menu_tabs_prev, TRUE);
 	gtk_action_set_sensitive (priv->menu_tabs_detach, num_pages > 1);
 	gtk_action_set_sensitive (priv->menu_tabs_left, !first_page);
 	gtk_action_set_sensitive (priv->menu_tabs_right, !last_page);
 	gtk_action_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
+}
+
+static void
+chat_window_contact_menu_update (EmpathyChatWindowPriv *priv,
+				 EmpathyChatWindow     *window)
+{
+	GtkWidget *menu, *submenu, *orig_submenu;
 
-	/* Update Contact menu */
 	menu = gtk_ui_manager_get_widget (priv->ui_manager,
 		"/chats_menubar/menu_contact");
 	orig_submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
+
+	DEBUG ("Update window : Contact Menu");
+
 	if (orig_submenu == NULL || !GTK_WIDGET_VISIBLE (orig_submenu)) {
 		submenu = empathy_chat_get_contact_menu (priv->current_chat);
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu), submenu);
@@ -359,17 +354,31 @@ chat_window_update (EmpathyChatWindow *window)
 					     (GCallback)_submenu_notify_visible_changed_cb,
 					     G_OBJECT (window));
 	}
+}
 
-	/* Update window title */
-	if (n_chats == 1) {
-		gtk_window_set_title (GTK_WINDOW (priv->dialog), name);
-	} else {
-		gchar *title;
+static void
+chat_window_title_update (EmpathyChatWindowPriv *priv)
+{
+	const gchar *name;
 
-		title = g_strdup_printf (_("Conversations (%d)"), n_chats);
-		gtk_window_set_title (GTK_WINDOW (priv->dialog), title);
-		g_free (title);
-	}
+	name = empathy_chat_get_name (priv->current_chat);
+
+	DEBUG ("Update window : Title");
+
+	gtk_window_set_title (GTK_WINDOW (priv->dialog), name);
+}
+
+static void
+chat_window_icon_update (EmpathyChatWindowPriv *priv)
+{
+	GdkPixbuf      *icon;
+	EmpathyContact *remote_contact;
+	gboolean        avatar_in_icon;
+	guint           n_chats;
+
+	n_chats = g_list_length (priv->chats);
+
+	DEBUG ("Update window : Icon");
 
 	/* Update window icon */
 	if (priv->chats_new_msg) {
@@ -392,6 +401,17 @@ chat_window_update (EmpathyChatWindow *window)
 			gtk_window_set_icon_name (GTK_WINDOW (priv->dialog), NULL);
 		}
 	}
+}
+
+static void
+chat_window_close_button_update (EmpathyChatWindowPriv *priv,
+				 gint num_pages)
+{
+	GtkWidget *chat;
+	GtkWidget *chat_close_button;
+	gint       i;
+
+	DEBUG ("Update window : Close Button");
 
 	if (num_pages == 1) {
 		chat = gtk_notebook_get_nth_page (GTK_NOTEBOOK (priv->notebook), 0);
@@ -408,6 +428,31 @@ chat_window_update (EmpathyChatWindow *window)
 	}
 }
 
+static void
+chat_window_update (EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv = GET_PRIV (window);
+	gint                   num_pages;
+
+	num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
+
+	DEBUG ("Update window");
+
+	/* Update Tab menu */
+	chat_window_menu_context_update (priv,
+					 num_pages);
+
+	chat_window_contact_menu_update (priv,
+					 window);
+
+	chat_window_title_update (priv);
+
+	chat_window_icon_update (priv);
+
+	chat_window_close_button_update (priv,
+					 num_pages);
+}
+
 static void
 append_markup_printf (GString    *string,
 		      const char *format,
@@ -433,7 +478,7 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	EmpathyContact        *remote_contact;
 	const gchar           *name;
 	const gchar           *id;
-	EmpathyAccount        *account;
+	TpAccount             *account;
 	const gchar           *subject;
 	const gchar           *status = NULL;
 	GtkWidget             *widget;
@@ -456,7 +501,7 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	remote_contact = empathy_chat_get_remote_contact (chat);
 
 	DEBUG ("Updating chat tab, name=%s, account=%s, subject=%s, remote_contact=%p",
-		name, empathy_account_get_unique_name (account), subject, remote_contact);
+		name, tp_proxy_get_object_path (account), subject, remote_contact);
 
 	/* Update tab image */
 	if (empathy_chat_get_tp_chat (chat) == NULL) {
@@ -500,7 +545,7 @@ chat_window_update_chat_tab (EmpathyChat *chat)
 	append_markup_printf (tooltip,
 			      "<b>%s</b><small> (%s)</small>",
 			      id,
-			      empathy_account_get_display_name (account));
+			      tp_account_get_display_name (account));
 
 	if (!EMP_STR_EMPTY (status)) {
 		append_markup_printf (tooltip, "\n<i>%s</i>", status);
@@ -595,7 +640,7 @@ chat_window_conv_activate_cb (GtkAction         *action,
 	is_room = empathy_chat_is_room (priv->current_chat);
 	if (is_room) {
 		const gchar *room;
-		EmpathyAccount   *account;
+		TpAccount   *account;
 		gboolean     found = FALSE;
 		EmpathyChatroom *chatroom;
 
@@ -644,7 +689,7 @@ chat_window_favorite_toggled_cb (GtkToggleAction   *toggle_action,
 {
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean               active;
-	EmpathyAccount        *account;
+	TpAccount             *account;
 	const gchar           *room;
 	EmpathyChatroom       *chatroom;
 
@@ -679,63 +724,6 @@ chat_window_contacts_toggled_cb (GtkToggleAction   *toggle_action,
 	empathy_chat_set_show_contacts (priv->current_chat, active);
 }
 
-static const gchar *
-chat_get_window_id_for_geometry (EmpathyChat *chat)
-{
-	const gchar *res = NULL;
-	gboolean     separate_windows;
-
-	empathy_conf_get_bool (empathy_conf_get (),
-			       EMPATHY_PREFS_UI_SEPARATE_CHAT_WINDOWS,
-			       &separate_windows);
-
-	if (separate_windows) {
-		res = empathy_chat_get_id (chat);
-	}
-
-	return res ? res : "chat-window";
-}
-
-static gboolean
-chat_window_save_geometry_timeout_cb (EmpathyChatWindow *window)
-{
-	EmpathyChatWindowPriv *priv;
-	gint                  x, y, w, h;
-
-	priv = GET_PRIV (window);
-
-	gtk_window_get_size (GTK_WINDOW (priv->dialog), &w, &h);
-	gtk_window_get_position (GTK_WINDOW (priv->dialog), &x, &y);
-
-	empathy_geometry_save (chat_get_window_id_for_geometry (priv->current_chat),
-			       x, y, w, h);
-
-	priv->save_geometry_id = 0;
-
-	return FALSE;
-}
-
-static gboolean
-chat_window_configure_event_cb (GtkWidget         *widget,
-				GdkEventConfigure *event,
-				EmpathyChatWindow  *window)
-{
-	EmpathyChatWindowPriv *priv;
-
-	priv = GET_PRIV (window);
-
-	if (priv->save_geometry_id != 0) {
-		g_source_remove (priv->save_geometry_id);
-	}
-
-	priv->save_geometry_id =
-		g_timeout_add_seconds (1,
-				       (GSourceFunc) chat_window_save_geometry_timeout_cb,
-				       window);
-
-	return FALSE;
-}
-
 static void
 chat_window_close_activate_cb (GtkAction         *action,
 			       EmpathyChatWindow *window)
@@ -826,6 +814,50 @@ chat_window_paste_activate_cb (GtkAction         *action,
 	empathy_chat_paste (priv->current_chat);
 }
 
+static void
+chat_window_tabs_next_activate_cb (GtkAction         *action,
+				   EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv;
+	EmpathyChat           *chat;
+	gint                  index_, numPages;
+
+	priv = GET_PRIV (window);
+
+	chat = priv->current_chat;
+	index_ = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
+	numPages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
+
+	if (index_ == (numPages - 1)) {
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), 0);
+		return;
+	}
+
+	gtk_notebook_next_page (GTK_NOTEBOOK (priv->notebook));
+}
+
+static void
+chat_window_tabs_previous_activate_cb (GtkAction         *action,
+				   EmpathyChatWindow *window)
+{
+	EmpathyChatWindowPriv *priv;
+	EmpathyChat           *chat;
+	gint                  index_, numPages;
+
+	priv = GET_PRIV (window);
+
+	chat = priv->current_chat;
+	index_ = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
+	numPages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
+
+	if (index_ <= 0) {
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), numPages - 1);
+		return;
+	}
+
+	gtk_notebook_prev_page (GTK_NOTEBOOK (priv->notebook));
+}
+
 static void
 chat_window_tabs_left_activate_cb (GtkAction         *action,
 				   EmpathyChatWindow *window)
@@ -990,7 +1022,7 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 	EmpathyChatWindowPriv *priv = GET_PRIV (window);
 	gboolean res;
 
-	if (!empathy_notification_is_enabled ()) {
+	if (!empathy_notify_manager_notification_is_enabled (priv->notify_mgr)) {
 		return;
 	} else {
 		empathy_conf_get_bool (empathy_conf_get (),
@@ -1022,7 +1054,8 @@ chat_window_show_or_update_notification (EmpathyChatWindow *window,
 				  G_CALLBACK (chat_window_notification_closed_cb), cb_data);
 	}
 
-	pixbuf = empathy_misc_get_pixbuf_for_notification (sender, EMPATHY_IMAGE_NEW_MESSAGE);
+	pixbuf = empathy_notify_manager_get_pixbuf_for_notification (priv->notify_mgr,
+		sender, EMPATHY_IMAGE_NEW_MESSAGE);
 
 	if (pixbuf != NULL) {
 		notify_notification_set_icon_from_pixbuf (priv->notification, pixbuf);
@@ -1301,15 +1334,18 @@ chat_window_drag_data_received (GtkWidget        *widget,
 	if (info == DND_DRAG_TYPE_CONTACT_ID) {
 		EmpathyChat           *chat = NULL;
 		EmpathyChatWindow     *old_window;
-		EmpathyAccount        *account = NULL;
-		EmpathyAccountManager *account_manager;
+		TpAccount             *account = NULL;
+		TpAccountManager      *account_manager;
 		const gchar           *id;
 		gchar                **strv;
 		const gchar           *account_id;
 		const gchar           *contact_id;
 
 		id = (const gchar*) gtk_selection_data_get_data (selection);
-		account_manager = empathy_account_manager_dup_singleton ();
+
+		/* FIXME: Perhaps should be sure that the account manager is
+		 * prepared before calling _ensure_account on it. */
+		account_manager = tp_account_manager_dup ();
 
 		DEBUG ("DND contact from roster with id:'%s'", id);
 
@@ -1318,7 +1354,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 			account_id = strv[0];
 			contact_id = strv[1];
 			account =
-				empathy_account_manager_get_account (account_manager, account_id);
+				tp_account_manager_ensure_account (account_manager, account_id);
 			if (account != NULL)
 				chat = empathy_chat_window_find_chat (account, contact_id);
 		}
@@ -1332,7 +1368,7 @@ chat_window_drag_data_received (GtkWidget        *widget,
 		if (!chat) {
 			TpConnection *connection;
 
-			connection = empathy_account_get_connection (account);
+			connection = tp_account_get_connection (account);
 
 			if (connection) {
 				empathy_dispatcher_chat_with_contact_id (
@@ -1416,9 +1452,7 @@ chat_window_finalize (GObject *object)
 
 	g_object_unref (priv->ui_manager);
 	g_object_unref (priv->chatroom_manager);
-	if (priv->save_geometry_id != 0) {
-		g_source_remove (priv->save_geometry_id);
-	}
+	g_object_unref (priv->notify_mgr);
 
 	if (priv->notification != NULL) {
 		notify_notification_close (priv->notification, NULL);
@@ -1495,7 +1529,6 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	g_free (filename);
 
 	empathy_builder_connect (gui, window,
-			      "chat_window", "configure-event", chat_window_configure_event_cb,
 			      "menu_conv", "activate", chat_window_conv_activate_cb,
 			      "menu_conv_clear", "activate", chat_window_clear_activate_cb,
 			      "menu_conv_favorite", "toggled", chat_window_favorite_toggled_cb,
@@ -1505,6 +1538,8 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			      "menu_edit_cut", "activate", chat_window_cut_activate_cb,
 			      "menu_edit_copy", "activate", chat_window_copy_activate_cb,
 			      "menu_edit_paste", "activate", chat_window_paste_activate_cb,
+			      "menu_tabs_next", "activate", chat_window_tabs_next_activate_cb,
+			      "menu_tabs_prev", "activate", chat_window_tabs_previous_activate_cb,
 			      "menu_tabs_left", "activate", chat_window_tabs_left_activate_cb,
 			      "menu_tabs_right", "activate", chat_window_tabs_right_activate_cb,
 			      "menu_tabs_detach", "activate", chat_window_detach_activate_cb,
@@ -1559,16 +1594,6 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 			  "delete_event",
 			  G_CALLBACK (chat_window_delete_event_cb),
 			  window);
-
-	g_signal_connect_swapped (priv->menu_tabs_prev,
-				  "activate",
-				  G_CALLBACK (gtk_notebook_prev_page),
-				  priv->notebook);
-	g_signal_connect_swapped (priv->menu_tabs_next,
-				  "activate",
-				  G_CALLBACK (gtk_notebook_next_page),
-				  priv->notebook);
-
 	g_signal_connect (priv->dialog,
 			  "focus_in_event",
 			  G_CALLBACK (chat_window_focus_in_event_cb),
@@ -1605,6 +1630,8 @@ empathy_chat_window_init (EmpathyChatWindow *window)
 	priv->chats_new_msg = NULL;
 	priv->chats_composing = NULL;
 	priv->current_chat = NULL;
+
+	priv->notify_mgr = empathy_notify_manager_dup_singleton ();
 }
 
 EmpathyChatWindow *
@@ -1668,7 +1695,6 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 	GtkWidget             *label;
 	GtkWidget             *popup_label;
 	GtkWidget             *child;
-	gint                   x, y, w, h;
 
 	g_return_if_fail (window != NULL);
 	g_return_if_fail (EMPATHY_IS_CHAT (chat));
@@ -1680,21 +1706,18 @@ empathy_chat_window_add_chat (EmpathyChatWindow *window,
 
 	/* If this window has just been created, position it */
 	if (priv->chats == NULL) {
-		empathy_geometry_load (chat_get_window_id_for_geometry (chat), &x, &y, &w, &h);
+		const gchar *name = "chat-window";
+		gboolean     separate_windows;
 
-		if (x >= 0 && y >= 0) {
-			/* Let the window manager position it if we don't have
-			 * good x, y coordinates.
-			 */
-			gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
+		empathy_conf_get_bool (empathy_conf_get (),
+				       EMPATHY_PREFS_UI_SEPARATE_CHAT_WINDOWS,
+				       &separate_windows);
+
+		if (separate_windows) {
+			name = empathy_chat_get_id (chat);
 		}
 
-		if (w > 0 && h > 0) {
-			/* Use the defaults from the ui file if we don't have
-			 * good w, h geometry.
-			 */
-			gtk_window_resize (GTK_WINDOW (priv->dialog), w, h);
-		}
+		empathy_geometry_bind (GTK_WINDOW (priv->dialog), name);
 	}
 
 	child = GTK_WIDGET (chat);
@@ -1818,7 +1841,7 @@ empathy_chat_window_has_focus (EmpathyChatWindow *window)
 }
 
 EmpathyChat *
-empathy_chat_window_find_chat (EmpathyAccount   *account,
+empathy_chat_window_find_chat (TpAccount   *account,
 			       const gchar *id)
 {
 	GList *l;
