@@ -114,22 +114,22 @@ log_level_to_string (guint level)
   switch (level)
     {
     case EMP_DEBUG_LEVEL_ERROR:
-      return _("Error");
+      return "Error";
       break;
     case EMP_DEBUG_LEVEL_CRITICAL:
-      return _("Critical");
+      return "Critical";
       break;
     case EMP_DEBUG_LEVEL_WARNING:
-      return _("Warning");
+      return "Warning";
       break;
     case EMP_DEBUG_LEVEL_MESSAGE:
-      return _("Message");
+      return "Message";
       break;
     case EMP_DEBUG_LEVEL_INFO:
-      return _("Info");
+      return "Info";
       break;
     case EMP_DEBUG_LEVEL_DEBUG:
-      return _("Debug");
+      return "Debug";
       break;
     default:
       g_assert_not_reached ();
@@ -733,6 +733,13 @@ debug_window_fill_cm_chooser (EmpathyDebugWindow *debug_window)
   tp_list_connection_names (priv->dbus, debug_window_list_connection_names_cb,
       debug_window, NULL, NULL);
 
+  /* add Mission Control */
+  gtk_list_store_append (priv->cms, &iter);
+  gtk_list_store_set (priv->cms, &iter,
+      COL_CM_NAME, "misson-control",
+      COL_CM_UNIQUE_NAME, "org.freedesktop.Telepathy.MissionControl5",
+      -1);
+
   priv->name_owner_changed_signal =
       tp_cli_dbus_daemon_connect_to_name_owner_changed (priv->dbus,
       debug_window_name_owner_changed_cb, debug_window, NULL, NULL, NULL);
@@ -888,6 +895,48 @@ debug_window_button_press_event_cb (GtkTreeView *view,
   return FALSE;
 }
 
+static gchar *
+debug_window_format_timestamp (gdouble timestamp)
+{
+  struct tm *tstruct;
+  char time_str[32];
+  gint ms;
+  time_t sec;
+  gchar *text;
+
+  ms = (int) ((timestamp - (int) timestamp)*1e6);
+  sec = (long) timestamp;
+  tstruct = localtime ((time_t *) &sec);
+  if (!strftime (time_str, sizeof (time_str), "%x %T", tstruct))
+    {
+      DEBUG ("Failed to format timestamp: %e", timestamp);
+      time_str[0] = '\0';
+    }
+
+  text = g_strdup_printf ("%s.%d", time_str, ms);
+
+  return text;
+}
+
+static void
+debug_window_time_formatter (GtkTreeViewColumn *tree_column,
+    GtkCellRenderer *cell,
+    GtkTreeModel *tree_model,
+    GtkTreeIter *iter,
+    gpointer data)
+{
+  gdouble timestamp;
+  gchar *time_str;
+
+  gtk_tree_model_get (tree_model, iter, COL_DEBUG_TIMESTAMP, &timestamp, -1);
+
+  time_str = debug_window_format_timestamp (timestamp);
+
+  g_object_set (G_OBJECT (cell), "text", time_str, NULL);
+
+  g_free (time_str);
+}
+
 static gboolean
 debug_window_store_filter_foreach (GtkTreeModel *model,
     GtkTreePath *path,
@@ -897,7 +946,7 @@ debug_window_store_filter_foreach (GtkTreeModel *model,
   GFileOutputStream *output_stream = (GFileOutputStream *) user_data;
   gchar *domain, *category, *message, *level_str, *level_upper;
   gdouble timestamp;
-  gchar *line;
+  gchar *line, *time_str;
   GError *error = NULL;
   gboolean out = FALSE;
 
@@ -911,9 +960,13 @@ debug_window_store_filter_foreach (GtkTreeModel *model,
 
   level_upper = g_ascii_strup (level_str, -1);
 
-  line = g_strdup_printf ("%s%s%s-%s: %e: %s\n",
+  time_str = debug_window_format_timestamp (timestamp);
+
+  line = g_strdup_printf ("%s%s%s-%s: %s: %s\n",
       domain, EMP_STR_EMPTY (category) ? "" : "/",
-      category, level_upper, timestamp, message);
+      category, level_upper, time_str, message);
+
+  g_free (time_str);
 
   g_output_stream_write (G_OUTPUT_STREAM (output_stream), line,
       strlen (line), NULL, &error);
@@ -1013,7 +1066,7 @@ debug_window_copy_model_foreach (GtkTreeModel *model,
   gchar *tmp;
   gchar *domain, *category, *message, *level_str, *level_upper;
   gdouble timestamp;
-  gchar *line;
+  gchar *line, *time_str;
 
   gtk_tree_model_get (model, iter,
       COL_DEBUG_TIMESTAMP, &timestamp,
@@ -1025,9 +1078,13 @@ debug_window_copy_model_foreach (GtkTreeModel *model,
 
   level_upper = g_ascii_strup (level_str, -1);
 
-  line = g_strdup_printf ("%s%s%s-%s: %e: %s\n",
+  time_str = debug_window_format_timestamp (timestamp);
+
+  line = g_strdup_printf ("%s%s%s-%s: %s: %s\n",
       domain, EMP_STR_EMPTY (category) ? "" : "/",
-      category, level_upper, timestamp, message);
+      category, level_upper, time_str, message);
+
+  g_free (time_str);
 
   tmp = g_strconcat (*text, line, NULL);
 
@@ -1263,8 +1320,9 @@ debug_window_constructor (GType type,
   renderer = gtk_cell_renderer_text_new ();
   g_object_set (renderer, "yalign", 0, NULL);
 
-  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->view),
-      -1, _("Time"), renderer, "text", COL_DEBUG_TIMESTAMP, NULL);
+  gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (priv->view),
+      -1, _("Time"), renderer,
+      (GtkTreeCellDataFunc) debug_window_time_formatter, NULL, NULL);
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->view),
       -1, _("Domain"), renderer, "text", COL_DEBUG_DOMAIN, NULL);
   gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->view),
@@ -1432,7 +1490,7 @@ empathy_debug_window_class_init (EmpathyDebugWindowClass *klass)
 GtkWidget *
 empathy_debug_window_new (GtkWindow *parent)
 {
-  g_return_val_if_fail (GTK_IS_WINDOW (parent), NULL);
+  g_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), NULL);
 
   return GTK_WIDGET (g_object_new (EMPATHY_TYPE_DEBUG_WINDOW,
       "transient-for", parent, NULL));
