@@ -24,11 +24,12 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/interfaces.h>
 
-#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-dispatcher.h>
+#include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-tp-contact-factory.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-tp-chat.h>
@@ -186,15 +187,22 @@ autoremove_event_timeout_cb (EventPriv *event)
 }
 
 static void
-event_manager_add (EmpathyEventManager *manager, EmpathyContact *contact,
-  const gchar *icon_name, const gchar *header, const gchar *message,
-  EventManagerApproval *approval, EventFunc func, gpointer user_data)
+event_manager_add (EmpathyEventManager *manager,
+    EmpathyContact *contact,
+    EmpathyEventType type,
+    const gchar *icon_name,
+    const gchar *header,
+    const gchar *message,
+    EventManagerApproval *approval,
+    EventFunc func,
+    gpointer user_data)
 {
   EmpathyEventManagerPriv *priv = GET_PRIV (manager);
   EventPriv               *event;
 
   event = g_slice_new0 (EventPriv);
   event->public.contact = contact ? g_object_ref (contact) : NULL;
+  event->public.type = type;
   event->public.icon_name = g_strdup (icon_name);
   event->public.header = g_strdup (header);
   event->public.message = g_strdup (message);
@@ -330,14 +338,14 @@ event_channel_process_voip_func (EventPriv *event)
 
   button = gtk_dialog_add_button (GTK_DIALOG (dialog),
       _("_Reject"), GTK_RESPONSE_REJECT);
-  image = gtk_image_new_from_icon_name (GTK_STOCK_CANCEL,
+  image = gtk_image_new_from_icon_name ("call-stop",
     GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
 
   button = gtk_dialog_add_button (GTK_DIALOG (dialog),
       _("_Answer"), GTK_RESPONSE_ACCEPT);
 
-  image = gtk_image_new_from_icon_name (GTK_STOCK_APPLY, GTK_ICON_SIZE_BUTTON);
+  image = gtk_image_new_from_icon_name ("call-start", GTK_ICON_SIZE_BUTTON);
   gtk_button_set_image (GTK_BUTTON (button), image);
 
   g_signal_connect (dialog, "response",
@@ -372,8 +380,9 @@ event_manager_chat_message_received_cb (EmpathyTpChat *tp_chat,
   if (event != NULL)
     event_update (approval->manager, event, EMPATHY_IMAGE_NEW_MESSAGE, header, msg);
   else
-    event_manager_add (approval->manager, sender, EMPATHY_IMAGE_NEW_MESSAGE, header,
-      msg, approval, event_text_channel_process_func, NULL);
+    event_manager_add (approval->manager, sender, EMPATHY_EVENT_TYPE_CHAT,
+        EMPATHY_IMAGE_NEW_MESSAGE, header, msg, approval,
+        event_text_channel_process_func, NULL);
 
   empathy_sound_play (empathy_main_window_get (),
     EMPATHY_SOUND_CONVERSATION_NEW);
@@ -446,9 +455,9 @@ event_manager_media_channel_got_contact (EventManagerApproval *approval)
   header = g_strdup_printf (_("Incoming call from %s"),
     empathy_contact_get_name (approval->contact));
 
-  event_manager_add (approval->manager,
-    approval->contact, EMPATHY_IMAGE_VOIP, header, NULL,
-    approval, event_channel_process_voip_func, NULL);
+  event_manager_add (approval->manager, approval->contact,
+      EMPATHY_EVENT_TYPE_VOIP, EMPATHY_IMAGE_VOIP, header, NULL, approval,
+      event_channel_process_voip_func, NULL);
 
   g_free (header);
 
@@ -502,8 +511,9 @@ event_manager_add_tube_approval (EventManagerApproval *approval,
               "application to handle it.");
     }
 
-  event_manager_add (approval->manager, approval->contact, icon_name, header,
-    msg, approval, event_manager_tube_approved_cb, approval);
+  event_manager_add (approval->manager, approval->contact,
+      EMPATHY_EVENT_TYPE_TUBE, icon_name, header, msg, approval,
+      event_manager_tube_approved_cb, approval);
 
   g_free (header);
   /* FIXME better sound for incoming tubes ? */
@@ -684,9 +694,9 @@ event_manager_muc_invite_got_contact_cb (EmpathyTpContactFactory *factory,
       empathy_contact_get_name (approval->contact),
       tp_channel_get_identifier (channel));
 
-  event_manager_add (approval->manager,
-    approval->contact, EMPATHY_IMAGE_GROUP_MESSAGE, msg, invite_msg,
-    approval, event_room_channel_process_func, NULL);
+  event_manager_add (approval->manager, approval->contact,
+      EMPATHY_EVENT_TYPE_CHAT, EMPATHY_IMAGE_GROUP_MESSAGE, msg, invite_msg,
+      approval, event_room_channel_process_func, NULL);
 
   empathy_sound_play (empathy_main_window_get (),
     EMPATHY_SOUND_CONVERSATION_NEW);
@@ -710,8 +720,8 @@ event_manager_ft_got_contact_cb (EmpathyTpContactFactory *factory,
                             empathy_contact_get_name (approval->contact));
 
   event_manager_add (approval->manager, approval->contact,
-      EMPATHY_IMAGE_DOCUMENT_SEND, header, NULL, approval,
-      event_channel_process_func, NULL);
+      EMPATHY_EVENT_TYPE_TRANSFER, EMPATHY_IMAGE_DOCUMENT_SEND, header, NULL,
+      approval, event_channel_process_func, NULL);
 
   /* FIXME better sound for incoming file transfers ?*/
   empathy_sound_play (empathy_main_window_get (),
@@ -894,8 +904,9 @@ event_manager_pendings_changed_cb (EmpathyContactList  *list,
   else
     event_msg = NULL;
 
-  event_manager_add (manager, contact, GTK_STOCK_DIALOG_QUESTION, header,
-    event_msg, NULL, event_pending_subscribe_func, NULL);
+  event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_SUBSCRIPTION,
+      GTK_STOCK_DIALOG_QUESTION, header, event_msg, NULL,
+      event_pending_subscribe_func, NULL);
 
   g_free (event_msg);
   g_free (header);
@@ -908,13 +919,16 @@ event_manager_presence_changed_cb (EmpathyContactMonitor *monitor,
     TpConnectionPresenceType previous,
     EmpathyEventManager *manager)
 {
-  EmpathyAccount *account;
+  TpAccount *account;
   gchar *header = NULL;
   gboolean preference = FALSE;
+  EmpathyIdle *idle;
 
   account = empathy_contact_get_account (contact);
-  if (empathy_account_is_just_connected (account))
-    return;
+  idle = empathy_idle_dup_singleton ();
+
+  if (empathy_idle_account_is_just_connected (idle, account))
+    goto out;
 
   if (tp_connection_presence_type_cmp_availability (previous,
      TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
@@ -929,8 +943,8 @@ event_manager_presence_changed_cb (EmpathyContactMonitor *monitor,
           header = g_strdup_printf (_("%s is now offline."),
             empathy_contact_get_name (contact));
 
-          event_manager_add (manager, contact, GTK_STOCK_DIALOG_INFO, header,
-                             NULL, NULL, NULL, NULL);
+          event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
+              GTK_STOCK_DIALOG_INFO, header, NULL, NULL, NULL, NULL);
         }
     }
   else
@@ -945,11 +959,14 @@ event_manager_presence_changed_cb (EmpathyContactMonitor *monitor,
           header = g_strdup_printf (_("%s is now online."),
             empathy_contact_get_name (contact));
 
-          event_manager_add (manager, contact, GTK_STOCK_DIALOG_INFO, header,
-                             NULL, NULL, NULL, NULL);
+          event_manager_add (manager, contact, EMPATHY_EVENT_TYPE_PRESENCE,
+              GTK_STOCK_DIALOG_INFO, header, NULL, NULL, NULL, NULL);
         }
     }
   g_free (header);
+
+out:
+  g_object_unref (idle);
 }
 
 
