@@ -32,23 +32,23 @@
 #include <libnotify/notification.h>
 #include <libnotify/notify.h>
 
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-idle.h>
-#include <libempathy/empathy-account-manager.h>
 
 #include <libempathy-gtk/empathy-presence-chooser.h>
 #include <libempathy-gtk/empathy-conf.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 #include <libempathy-gtk/empathy-images.h>
 #include <libempathy-gtk/empathy-new-message-dialog.h>
+#include <libempathy-gtk/empathy-notify-manager.h>
 
 #include "empathy-accounts-dialog.h"
 #include "empathy-status-icon.h"
 #include "empathy-preferences.h"
 #include "empathy-event-manager.h"
-#include "empathy-misc.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_DISPATCHER
 #include <libempathy/empathy-debug.h>
@@ -60,7 +60,8 @@
 typedef struct {
 	GtkStatusIcon       *icon;
 	EmpathyIdle         *idle;
-	EmpathyAccountManager *account_manager;
+	TpAccountManager    *account_manager;
+	EmpathyNotifyManager *notify_mgr;
 	gboolean             showing_event_icon;
 	guint                blink_timeout;
 	EmpathyEventManager *event_manager;
@@ -73,7 +74,6 @@ typedef struct {
 	GtkAction           *show_window_item;
 	GtkAction           *new_message_item;
 	GtkAction           *status_item;
-	gboolean             notify_supports_actions;
 } EmpathyStatusIconPriv;
 
 G_DEFINE_TYPE (EmpathyStatusIcon, empathy_status_icon, G_TYPE_OBJECT);
@@ -147,7 +147,7 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
 	GdkPixbuf *pixbuf = NULL;
 
-	if (!empathy_notification_is_enabled ()) {
+	if (!empathy_notify_manager_notification_is_enabled (priv->notify_mgr)) {
 		/* always close the notification if this happens */
 		notification_close_helper (priv);
 		return;
@@ -155,21 +155,26 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 
 	if (priv->event) {
 		gchar *message_esc = NULL;
+		gchar *header_esc = NULL;
 
 		if (priv->event->message != NULL)
 			message_esc = g_markup_escape_text (priv->event->message, -1);
 
+		if (priv->event->header != NULL)
+			header_esc = g_markup_escape_text (priv->event->header, -1);
+
 		if (priv->notification) {
 			notify_notification_update (priv->notification,
-						    priv->event->header, message_esc,
+						    header_esc, message_esc,
 						    NULL);
 		} else {
 			priv->notification = notify_notification_new_with_status_icon
-				(priv->event->header, message_esc, NULL, priv->icon);
+				(header_esc, message_esc, NULL, priv->icon);
 			notify_notification_set_timeout (priv->notification,
 							 NOTIFY_EXPIRES_DEFAULT);
 
-			if (priv->notify_supports_actions) {
+			if (empathy_notify_manager_has_capability (priv->notify_mgr,
+			           EMPATHY_NOTIFY_MANAGER_CAP_ACTIONS)) {
 				notify_notification_add_action (priv->notification,
 					"respond",
 					_("Respond"),
@@ -182,7 +187,8 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 					  G_CALLBACK (status_icon_notification_closed_cb), icon);
 		}
 
-		pixbuf = empathy_misc_get_pixbuf_for_notification (priv->event->contact,
+		pixbuf = empathy_notify_manager_get_pixbuf_for_notification (
+								   priv->notify_mgr, priv->event->contact,
 								   priv->event->icon_name);
 
 		if (pixbuf != NULL) {
@@ -194,6 +200,7 @@ status_icon_update_notification (EmpathyStatusIcon *icon)
 		notify_notification_show (priv->notification, NULL);
 
 		g_free (message_esc);
+		g_free (header_esc);
 	} else {
 		notification_close_helper (priv);
 	}
@@ -234,6 +241,13 @@ status_icon_update_icon (EmpathyStatusIcon *icon)
 		TpConnectionPresenceType state;
 
 		state = empathy_idle_get_state (priv->idle);
+
+		/* An unset presence type here doesn't make sense. Force it
+		 * to be offline. */
+		if (state == TP_CONNECTION_PRESENCE_TYPE_UNSET) {
+			state = TP_CONNECTION_PRESENCE_TYPE_OFFLINE;
+		}
+
 		icon_name = empathy_icon_name_for_presence (state);
 	}
 
@@ -317,7 +331,7 @@ status_icon_event_updated_cb (EmpathyEventManager *manager,
 		return;
 	}
 
-	if (empathy_notification_is_enabled ()) {
+	if (empathy_notify_manager_notification_is_enabled (priv->notify_mgr)) {
 		status_icon_update_notification (icon);
 	}
 
@@ -374,7 +388,7 @@ status_icon_idle_notify_cb (EmpathyStatusIcon *icon)
 	status_icon_update_icon (icon);
 	status_icon_update_tooltip (icon);
 
-	if (!empathy_notification_is_enabled ()) {
+	if (!empathy_notify_manager_notification_is_enabled (priv->notify_mgr)) {
 		/* dismiss the outstanding notification if present */
 
 		if (priv->notification) {
@@ -506,20 +520,18 @@ status_icon_create_menu (EmpathyStatusIcon *icon)
 }
 
 static void
-status_icon_connection_changed_cb (EmpathyAccountManager *manager,
-				   EmpathyAccount *account,
-				   TpConnectionStatusReason reason,
-				   TpConnectionStatus current,
-				   TpConnectionStatus previous,
-				   EmpathyStatusIcon *icon)
+status_icon_status_changed_cb (TpAccount *account,
+			       TpConnectionStatus current,
+			       TpConnectionStatus previous,
+			       TpConnectionStatusReason reason,
+			       gchar *dbus_error_name,
+			       GHashTable *details,
+			       EmpathyStatusIcon *icon)
 {
 	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
-	int connected_accounts;
 
-	/* Check for a connected account */
-	connected_accounts = empathy_account_manager_get_connected_accounts (manager);
-
-	gtk_action_set_sensitive (priv->new_message_item, connected_accounts > 0);
+	gtk_action_set_sensitive (priv->new_message_item,
+				  empathy_account_manager_get_accounts_connected (NULL));
 }
 
 static void
@@ -531,10 +543,6 @@ status_icon_finalize (GObject *object)
 		g_source_remove (priv->blink_timeout);
 	}
 
-	g_signal_handlers_disconnect_by_func (priv->account_manager,
-					      status_icon_connection_changed_cb,
-					      object);
-
 	if (priv->notification) {
 		notify_notification_close (priv->notification, NULL);
 		g_object_unref (priv->notification);
@@ -546,6 +554,7 @@ status_icon_finalize (GObject *object)
 	g_object_unref (priv->account_manager);
 	g_object_unref (priv->event_manager);
 	g_object_unref (priv->ui_manager);
+	g_object_unref (priv->notify_mgr);
 }
 
 static void
@@ -558,22 +567,45 @@ empathy_status_icon_class_init (EmpathyStatusIconClass *klass)
 	g_type_class_add_private (object_class, sizeof (EmpathyStatusIconPriv));
 }
 
+static void
+account_manager_prepared_cb (GObject *source_object,
+			     GAsyncResult *result,
+			     gpointer user_data)
+{
+	GList *list, *l;
+	TpAccountManager *account_manager = TP_ACCOUNT_MANAGER (source_object);
+	EmpathyStatusIcon *icon = user_data;
+	GError *error = NULL;
+
+	if (!tp_account_manager_prepare_finish (account_manager, result, &error)) {
+		DEBUG ("Failed to prepare account manager: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	list = tp_account_manager_get_valid_accounts (account_manager);
+	for (l = list; l != NULL; l = l->next) {
+		empathy_signal_connect_weak (l->data, "status-changed",
+					     G_CALLBACK (status_icon_status_changed_cb),
+					     G_OBJECT (icon));
+	}
+	g_list_free (list);
+}
+
 static void
 empathy_status_icon_init (EmpathyStatusIcon *icon)
 {
 	EmpathyStatusIconPriv *priv = G_TYPE_INSTANCE_GET_PRIVATE (icon,
 		EMPATHY_TYPE_STATUS_ICON, EmpathyStatusIconPriv);
-	GList *list, *l;
 
 	icon->priv = priv;
 	priv->icon = gtk_status_icon_new ();
-	priv->account_manager = empathy_account_manager_dup_singleton ();
+	priv->account_manager = tp_account_manager_dup ();
 	priv->idle = empathy_idle_dup_singleton ();
 	priv->event_manager = empathy_event_manager_dup_singleton ();
 
-	g_signal_connect (priv->account_manager,
-			  "account-connection-changed",
-			  G_CALLBACK (status_icon_connection_changed_cb), icon);
+	tp_account_manager_prepare_async (priv->account_manager, NULL,
+	    account_manager_prepared_cb, icon);
 
 	/* make icon listen and respond to MAIN_WINDOW_HIDDEN changes */
 	empathy_conf_notify_add (empathy_conf_get (),
@@ -603,16 +635,7 @@ empathy_status_icon_init (EmpathyStatusIcon *icon)
 			  G_CALLBACK (status_icon_popup_menu_cb),
 			  icon);
 
-	/* Query the notification daemon to check if it supports actions */
-	priv->notify_supports_actions = FALSE;
-	list = notify_get_server_caps ();
-	for (l = list; l != NULL; l = g_list_next (l)) {
-		gchar *caps = l->data;
-		if (!tp_strdiff (caps, "actions"))
-			priv->notify_supports_actions = TRUE;
-		g_free (caps);
-	}
-	g_list_free (list);
+	priv->notify_mgr = empathy_notify_manager_dup_singleton ();
 }
 
 EmpathyStatusIcon *
